


;-----------------------------------------------------------


;This Widescreen Patch was designed with the 16:9 Aspect Ratio in mind.
;Play your patched game in bsnes-hd beta emulator.


;-----------------------------------------------------------


;Credits:

;Patrick Johnston: Super Metroid disassembly
;http://patrickjohnston.org/ASM/ROM%20data/Super%20Metroid/

;Kazuto: Blackout Speedups (Doors Transitions Speed, Pause Screen Speed & Door Centering Speed)
;http://metroidconstruction.com/resource.php?id=75

;Lioran: Elevator speed change
;http://metroidconstruction.com/resource.php?id=104

;ocesse: Super Metroid - Widescreen Patch v0.2.1 asm
;https://www.youtube.com/user/ocesse/videos

;Special Thanks:

;byuu (near): bsnes developer
;https://byuu.org/bsnes

;byuu (near): xkas v0.6 developer (Please use this tool to apply the Widescreen Patch v0.2.1 asm to Super Metroid & hacks unheadered)
;https://www.romhacking.net/utilities/269/

;DerKoun: bsnes-hd beta developer
;https://github.com/DerKoun/bsnes-hd

;Brett Tabke: A 6502 Programmer's Introduction to the 65816 (Commodore World magazine, Issue #16) (Internet Archive)
;http://www.defence-force.org/computing/oric/coding/annexe_2/
;https://ia802900.us.archive.org/12/items/commodore-world-16/Commodore_World_Issue_16.pdf


;-----------------------------------------------------------


lorom

 ; DEFINITIONS

; Safe to move anywhere in Bank
; Scrolling Subroutines
!7200 = $F200      ;$F200  Free space required: DA0h Bytes, From 7200 to 7F9F in ROM Addresses
!7205 = !7200+$5   ;$F205
!7340 = !7200+$140 ;$F340
!7400 = !7200+$200 ;$F400
!74B0 = !7200+$2B0 ;$F4B0
!7520 = !7200+$320 ;$F520
!75A0 = !7200+$3A0 ;$F5A0
!7730 = !7200+$530 ;$F730
!7C10 = !7200+$A10 ;$FC10
!7E90 = !7200+$C90 ;$FE90
!7F90 = !7200+$D90 ;$FF90

; Safe to move anywhere in Bank
; Process animation of items, doors and blocks that respawn throughout the 16:9 area
!27E00 = $FE00     ;$84FE00  Free space required: Bh Bytes, From 27E00 to 27E0A in ROM Addresses

; Kazuto's - Doors Transitions Speed
;Door stuff
!S = $0008	;Door scroll speed, must be $0002, $0004 (default) or $0008
!F = $0006	;Screen fade speed, higher = slower ($000C default)
;Don't touch these values
!X = $0100/!S	;Horizontal loop counter, X times S should equal $100
!Y = $00E0/!S+1	;Vertical loop counter, Y times S should equal $E0 ($100-$20 due to the HUD)
!C = $0010/!S+1	;Vertical counter for drawing tile rows "behind" HUD (prior to scrolling)

; Kazuto's - Pause Screen Speed
;Pause stuff
!P = $0000 	;Higher fades slower ($0001 default)

; Kazuto's - Door Centering Speed
!Speed = #$02	;Pixels per-frame to slide the screen, default $01
!FreeSpace = $82FC60   ;$82FC60  Free space required: 31h Bytes, From 17C60 to 17C90 in ROM Addresses
;Super Metroid - Hyper Metroid v1.0 (Override code)
;!FreeSpace = $82F770	;Safe to move anywhere in ROM ($017770)
;Changed to
;!FreeSpace = $82FC60	;Safe to move anywhere in ROM ($017C60)

; Lioran's 2x Elevator Speed + Down fix
!velocity = $0002 ;changeing this will force you to change !timer aswell dont go higher than 8
!timer = $0024             ;$0C is good for velocity 5
			   ;$10 is good for velocity 4
			   ;$18 good for velocity 3,
			   ;$20 good for 2, I'm using 24
			   ;$if you see samus stick out the bottom, increase the value, if stick out the top, lower the value
; Safe to move anywhere in ROM
;Free space: !freebytes
;Bank $A3F320
;ROM 11F320
!freebytes = $A3FBC0 ;$A3FBC0  Free space required: 2Bh Bytes, From 11FBC0 to 11FBEA in ROM Addresses
!freeram = $0741 ;just some free ram that's used for the timer from bank 7E if you need to change it somehow.
;Super Metroid - Hyper Metroid v1.0 (Override code)
;Free space
;Bank $A3F320 Changed to $A3FBC0
;ROM  11F320 Changed to 11FBC0

; Safe to move anywhere in Bank
;Enemies Part 1
!85FF30 = $85FF00      ;$85FF00  Free space required: AEh Bytes, From 2FF00 to 2FFAD in ROM Addresses
!85FF50 = !85FF30+$20
!85FF70 = !85FF30+$40
!85FF90 = !85FF30+$60
!85FFB0 = !85FF30+$80
!85FFD0 = !85FF30+$A0
;Super Metroid - Hyper Metroid v1.0 (Override code)
;Free space
;Changed from !85FF30 = $85FF30
;        to   !85FF30 = $85FF00 (2FF00 B0h Bytes)

; Safe to move anywhere in Bank
;Enemies Part 2
!9FFF60 = $9FFF60      ;$9FFF60  Free space required: 91h Bytes, From FFF60 to FFFF0 in ROM Addresses
!9FFF90 = !9FFF60+$30
!9FFFB0 = !9FFF60+$50
!9FFFE0 = !9FFF60+$80


;-----------------------------------------------------------


; 7200 $80A9DE Update background data column Modified to load Black Blocks

; In a normal room of one screen wide (256 pixels), the Widescreen Patch cover the garbage tiles in the parts outside of the level,
; 5 columns of 16 tiles of 2x2 in both sides of the 16:9 screen, with 320 small black tiles per side.  
; I like call Black Blocks the 2x2 black tiles :) 

org !7200

LDX #$001C             ;  X = 1Ch
BRA $03                ;  Branch Always

org !7205

LDX #$0000             ;  X = 0
LDA $0783  ;[$7E:0783] ;\
BEQ $01                ;} If mode 7: return
RTS                    ;/
PHP                    ;
SEP #$20               ;\
LDA $07A5  ;[$7E:07A5] ;|
STA $4202  ;[$7E:4202] ;|
LDA $0992  ;[$7E:0992] ;|
STA $4203  ;[$7E:4203] ;|
PHB                    ;|
REP #$30               ;|
LDA $0990  ;[$7E:0990] ;|
CLC                    ;|
ADC $4216  ;[$7E:4216] ;|
ASL A                  ;} $36 = address of blocks to update (in bank $7F)
CLC                    ;|
ADC #$0002             ;|
TXY                    ;|
BEQ $04                ;|
CLC                    ;|
ADC #$9600             ;|
STA $36    ;[$7E:0036] ;|
LDA #$007F             ;|
STA $38    ;[$7E:0038] ;/
LDA $0996  ;[$7E:0996] ;\
ASL A                  ;|
ASL A                  ;} Wrapped tilemap VRAM update size = ([VRAM blocks to update Y block] & Fh) * 4
AND #$003C             ;|
STA $0958,x ;[$7E:0958];/
EOR #$003F             ;\
INC A                  ;} Unwrapped tilemap VRAM update size = (10h - ([VRAM blocks to update Y block] & Fh)) * 4
STA $0956,x ;[$7E:0956];/
SEP #$20               ;\
LDA $0996  ;[$7E:0996] ;|
AND #$0F               ;|
STA $4202  ;[$7E:4202] ;|
LDA #$40               ;|
STA $4203  ;[$7E:4203] ;|
REP #$20               ;|
LDA $0994  ;[$7E:0994] ;} VRAM offset of blocks to update = (([VRAM blocks to update Y block] & Fh) * 20h + ([X block of VRAM blocks to update] & 1Fh)) * 2
AND #$001F             ;|
STA $0935  ;[$7E:0935] ;|
ASL A                  ;|
CLC                    ;|
ADC $4216  ;[$7E:4216] ;|
STA $0933  ;[$7E:0933] ;/
LDA #$5000             ;  A = VRAM map 1st screen base address
LDY $0935  ;[$7E:0935] ;\
CPY #$0010             ;} If [X block of VRAM blocks to update] & 1Fh >= 10h:
BCC $03                ;/
LDA #$53E0             ;  A = VRAM map 2nd screen base address (devs opted to subtract 20h here, rather than take $0935 & Fh before summing)
TXY                    ;\
BEQ $04                ;} If background update:
SEC                    ;\
SBC $098E  ;[$7E:098E] ;} A -= [size of BG2]
STA $0937  ;[$7E:0937] ;  Base address of VRAM tilemap screen = [A]
CLC                    ;\
ADC $0933  ;[$7E:0933] ;} Unwrapped tilemap VRAM update destination = [base address of VRAM tilemap screen] + [VRAM offset of blocks to update]
STA $095A,x ;[$7E:095A];/
LDA $0937  ;[$7E:0937] ;\
CLC                    ;|
ADC $0935  ;[$7E:0935] ;} Wrapped tilemap VRAM update destination = [base address of VRAM tilemap screen] + ([X block of VRAM blocks to update] & 1Fh) * 2
ADC $0935  ;[$7E:0935] ;|
STA $095C,x ;[$7E:095C];/
LDA #$C8C8             ;\
LDY #$0000             ;|
CPX #$0000             ;|
BEQ $06                ;} New tilemap source address = $C8C8 if level update, else $C9D0 (background update)
LDA #$C9D0             ;|
LDY #$0108             ;/
CLC                    ;\
ADC $0956,x ;[$7E:0956];} Wrapped tilemap VRAM update left halves source = new tilemap source address + [unwrapped tilemap VRAM update size]
STA $095E,x ;[$7E:095E];/
CLC                    ;\
ADC #$0040             ;} Wrapped tilemap VRAM update right halves source = [wrapped tilemap VRAM update left halves source] + 40h
STA $0960,x ;[$7E:0960];/
STY $0937  ;[$7E:0937] ;  $0937 = index of VRAM tilemap source data
SEP #$20               ;\
LDA #$7E               ;|
PHA                    ;} DB = $7E
PLB                    ;|
REP #$20               ;/
PHX                    ;  Save X parameter
LDY #$0000             ;  Y = 0 (nth row offset for block to update)
LDA #$0010             ;\
STA $0939  ;[$7E:0939] ;} $0939 = 10h (counter for following loop)
LDA [$36],y ;[$7F:0002];\
STA $093B  ;[$7E:093B] ;} $093B = block to update (in $7F)
PHY                    ;  Push Y
LDA $079F  ;[$7E:079F] ;  Region Number  0000 Crateria, 0001 Brinstar, 0002 Norfair, 0003 Wrecked Ship, 0004 Maridia, 0005 Tourian, 0006 Ceres Station
CMP #$0006             ;  Compare A with #$0006  0006 Ceres Station
BNE $10                ;  BNE to  STA $C8C8,y ;[$7E:C8C8];  Store top-left VRAM tile of the block
LDA #$2803             ;  (2803 air block in Ceres Station, 00DC air block in Ridley's Room of Ceres Station, 2323 black block)    
LDY $079B  ;[$7E:079B] ;  Number of Room in current area
CPY #$E0B5             ;  Compare Y with #$E0B5  Ridley's Room in Ceres Station 
BNE $08                ; 
LDA #$00DC             ;  (2803 air block in Ceres Station, 00DC air block in Ridley's Room of Ceres Station, 2323 black block)
BRA $03                ;
LDA #$2323             ;  (2803 air block in Ceres Station, 00DC air block in Ridley's Room of Ceres Station, 2323 black block)
LDY $0937  ;[$7E:0937] ;  Y = [index of VRAM tilemap source data]
STA $C8C8,y ;[$7E:C8C8];  Store top-left VRAM tile of the block
STA $C908,y ;[$7E:C908];  Store top-right VRAM tile of the block
STA $C8CA,y ;[$7E:C8CA];  Store bottom-left VRAM tile of the block
STA $C90A,y ;[$7E:C90A];  Store bottom-right VRAM tile of the block
INY                    ;\
INY                    ;|
INY                    ;} Index of VRAM tilemap source data += 4
INY                    ;|
STY $0937  ;[$7E:0937] ;/
PLA                    ;\
CLC                    ;|
ADC $07A5  ;[$7E:07A5] ;} Y = next row offset
ADC $07A5  ;[$7E:07A5] ;|
TAY                    ;/
DEC $0939  ;[$7E:0939] ;  Decrement $0939
BEQ $02                ;\
BRA $B9                ;} If [$0939] != 0: go to LOOP
PLX                    ;  Restore X parameter
INC $0962,x ;[$7E:0962];  Set flag to perform VRAM update DMAs
PLB                    ;
PLP                    ;
RTS                    ;


;-----------------------------------------------------------


; 7340 Update BG graphics when scrolling Modified Left - Fast Doors

org !7340

LDA $0925  ;[$7E:0925] ; 
SEC                    ; 
SBC #$0005             ;  Substracting 5 from X, The values are as follow -4,-2,0,2,and,4 normal tiles    6,8,A,C,and,E black block
TAX                    ;  Transfer Acumulator to X register
TXA                    ;\
CLC                    ;|
ADC $08F7  ;[$7E:08F7] ;} Blocks to update X block = [layer 1 block X] + [X]
STA $0990  ;[$7E:0990] ;/
TXA                    ;\
CLC                    ;|
ADC $0907  ;[$7E:0907] ;} VRAM blocks to update X block = [BG1 block X] + [X]
STA $0994  ;[$7E:0994] ;/
LDA $08F9  ;[$7E:08F9] ;\
STA $0992  ;[$7E:0992] ;} Blocks to update Y block = [layer 1 block Y]
LDA $0909  ;[$7E:0909] ;\
STA $0996  ;[$7E:0996] ;} VRAM blocks to update Y block = [BG1 block Y]
CPX #$0006             ;\ Compare X with #$0006 The values are as follow -4,-2,0,2,and,4 normal tiles    6,8,A,C,and,E black block
BPL $05                ;} BPL Branch when is equal or greater, the comparison above
JSR $A9DB              ;  Update level data column Original
BRA $18                ;
LDA $07A8  ;[$7E:07A8] ;  $7E:07A8 Stores width of the room in pixels (100h,200h,300h,400h or more)
LSR A                  ;  Divide by 2
LSR A                  ;  Divide by 2
LSR A                  ;  Divide by 2
LSR A                  ;  Divide by 2
SEC                    ; 
SBC #$000B             ;  Substrat #$000Bh (0011d)
CMP $08F7  ;[$7E:08F7] ;  Compare A with tile of the room,   Position of block X Layer 1
BMI $05                ;  BMI Branch only when is minor, the comparison above
JSR $A9DB              ;  Update level data column Original
BRA $03                ;  BRA Branch always
JSR !7205              ;  Update level data column Modified to load Black Blocks
LDA $091B  ;[$7E:091B] ;\
LSR A                  ;} If not using custom layer 2: go to RTL
BCS $46    ;[RTL]      ;/ BCS $46
LDA $0925  ;[$7E:0925] ; 
SEC                    ;
SBC #$0005             ;  Substracting 5 from X, The values are as follow -4,-2,0,2,and,4 normal tiles    6,8,A,C,and,E black block
TAX                    ;  Transfer Acumulator to X register
TXA                    ;\
CLC                    ;|
ADC $08FB  ;[$7E:08FB] ;} Blocks to update X block = [layer 2 block X] + [X]
STA $0990  ;[$7E:0990] ;/
TXA                    ;\
CLC                    ;|
ADC $090B  ;[$7E:090B] ;} VRAM blocks to update X block = [BG2 block X] + [X]
STA $0994  ;[$7E:0994] ;/
LDA $08FD  ;[$7E:08FD] ;\
STA $0992  ;[$7E:0992] ;} Blocks to update Y block = [layer 2 block Y]
LDA $090D  ;[$7E:090D] ;\
STA $0996  ;[$7E:0996] ;} VRAM blocks to update Y block = [BG2 block Y]
CPX #$0006             ;  Compare X with #$0006
BPL $05                ; 
JSR $A9D6  ;[$80:A9D6] ;  Update background data column Original
BRA $18                ;
LDA $07A8  ;[$7E:07A8] ;  $7E:07A8 Stores width of the room in pixels (100h,200h,300h,400h or more)
LSR A                  ;  Divide by 2
LSR A                  ;  Divide by 2
LSR A                  ;  Divide by 2
LSR A                  ;  Divide by 2
SEC                    ; 
SBC #$000B             ;  Substrat #$000Bh (0011d)
CMP $08F7  ;[$7E:08F7] ;  Compare A with tile of the room,   Position of block X Layer 1
BMI $05                ; 
JSR $A9D6  ;[$80:A9D6] ;  Update background data column Original
BRA $03                ;
JSR !7200  ;[$80:F200] ;  Update background data column Modified to load Black Blocks
RTL                    ;


;-----------------------------------------------------------


; 7400 Update BG graphics when scrolling Modified Right - Fast Doors

org !7400

LDA #$0015             ;  Load 21d to Accumulator
SEC                    ;  Set Carry for Substraction
SBC $0925  ;[$7E:0925] ;  21-2,21-4,21-6,21-8,21-10,21-12,21-14,21-16,21-18,21-20, 1h-Bh= -10h  counting -5 0 +21  27 columns of tiles in all the screen del area 16:9
TAX                    ;  X=19,X=17,X=15,X=13,X=11,X=9,X=7,X=5,X=3,X=1,
TXA                    ;\ A=19,A=17,A=15,A=13,A=11,A=9,A=7,A=5,A=3,A=1,
CLC                    ;|
ADC $08F7  ;[$7E:08F7] ;} Blocks to update X block = [layer 1 block X] + [X]
STA $0990  ;[$7E:0990] ;/
TXA                    ;\
CLC                    ;| Clear Carry for Addition
ADC $0907  ;[$7E:0907] ;} VRAM blocks to update X block = [BG1 block X] + [X]
STA $0994  ;[$7E:0994] ;/
LDA $08F9  ;[$7E:08F9] ;\
STA $0992  ;[$7E:0992] ;} Blocks to update Y block = [layer 1 block Y]
LDA $0909  ;[$7E:0909] ;\
STA $0996  ;[$7E:0996] ;} VRAM blocks to update Y block = [BG1 block Y]
CPX #$000A             ;  13 with 0A/11 with 0A/0F with 0A/0D with 0A/0B with 0A/09 with 0A/07 with 0A/05 with 0A/03 with 0A/01 with 0A/
BMI $05                ;  Branch if Minus
JSR $A9DB              ;  $80A9DB: Update level data column Original
BRA $10                ;
LDA $08F7  ;[$7E:08F7] ;  FFF6,FFF7,FFF8,FFF9,FFFA   How many blocks X Layer 1 is scrolled
CMP #$FFFB             ;  FFFBh= -5d
BMI $05                ; 
JSR $A9DB              ;  $80A9DB: Update level data column Original
BRA $03                ;
JSR !7205              ;  $80F205: Update level data column Modified to load Black Blocks
LDA $091B  ;[$7E:091B] ;\
LSR A                  ;} If not using custom layer 2: go to RTL
BCS $3E    ;[RTL]      ;/ BCS $31
LDA #$0015             ;
SEC                    ; 
SBC $0925  ;[$7E:0925] ;  1h-Bh= -10h  counting -5 0 +21  27 columns of tiles in all the screen of the 16:9 area
TAX                    ;  Transfer Acumulator to X register
TXA                    ;\
CLC                    ;|
ADC $08FB  ;[$7E:08FB] ;} Blocks to update X block = [layer 2 block X] + [X]
STA $0990  ;[$7E:0990] ;/
TXA                    ;\
CLC                    ;|
ADC $090B  ;[$7E:090B] ;} VRAM blocks to update X block = [BG2 block X] + [X]
STA $0994  ;[$7E:0994] ;/
LDA $08FD  ;[$7E:08FD] ;\
STA $0992  ;[$7E:0992] ;} Blocks to update Y block = [layer 2 block Y]
LDA $090D  ;[$7E:090D] ;\
STA $0996  ;[$7E:0996] ;} VRAM blocks to update Y block = [BG2 block Y]
CPX #$000A             ;  15 with 0A/11 with 0A/0F with 0A/0D with 0A/0B with 0A/09 with 0A/07 with 0A/05 with 0A/03 with 0A/01 with 0A/
BMI $05                ; 
JSR $A9D6  ;[$80:A9D6] ;  $80A9D6: Update background data column Original
BRA $10                ;
LDA $08F7  ;[$7E:08F7] ;  FFF6,FFF7,FFF8,FFF9,FFFA   How many blocks X Layer 1 is scrolled
CMP #$FFFB             ;  FFFBh= -5d
BMI $05                ; 
JSR $A9D6  ;[$80:A9D6] ;  $80A9D6: Update background data column Original
BRA $03                ;
JSR !7200  ;[$80:F200] ;  $80F200: Update background data column Modified to load Black Blocks
RTL                    ;


;-----------------------------------------------------------


; 74B0 $80AEC2 Door transition - left Modified - Fast Doors

org !74B0

; $80AEC2 Door transition - left modified to executes $80A3A0 Calculate BG scrolls and update BG graphics when scrolling Original or 7340 Update BG
; graphics when scrolling Modified Left

LDX $0925  ;[$7E:0925] ;\
PHX                    ;} Save number of times screen has scrolled
LDA $0AF8  ;[$7E:0AF8] ;\
SEC                    ;|
SBC $092B  ;[$7E:092B] ;} Samus X sub-position -= door transition sub-speed
STA $0AF8  ;[$7E:0AF8] ;/
LDA $0AF6  ;[$7E:0AF6] ;\
SBC $092D  ;[$7E:092D] ;} Samus X position -= door transition speed
STA $0AF6  ;[$7E:0AF6] ;/
STA $0B10  ;[$7E:0B10] ;  Samus previous X position = Samus X position
LDA $0911  ;[$7E:0911] ;\
SEC                    ;|
SBC #$0008             ;} Layer 1 X position -= 8 I changed the value from 4 to 8 for Fast Doors
STA $0911  ;[$7E:0911] ;/
LDA $0917  ;[$7E:0917] ;\
SEC                    ;|
SBC #$0008             ;} Layer 2 X position -= 8 I changed the value from 4 to 8 for Fast Doors
STA $0917  ;[$7E:0917] ;/
CPX #$0014             ;  Decimal values 19,17,15,13,11,9,7,5,3,and,1 execute $80A3DF: Update BG graphics when scrolling Modified Left Black Blocks
BPL $0A                ;  Branch if Plus or Equal (10d Bytes)
LDA $0925  ;[$7E:0925] ;  number of times screen has scrolled
LSR A                  ;  Divide by 2, even number C-carry clear, non number C-carry set 
BCC $04                ;  Branch on Carry Clear to 22 A0 A3 80 JSL $80A3A0 Calculate BG scrolls and update BG graphics when scrolling Original  
JSL $800000+!7340 ;[$80:F340];  $80A3DF: Update BG graphics when scrolling Modified Left - Fast Doors
JSL $80A3A0 ;[$80:A3A0];  $80A3A0 Calculate BG scrolls and update BG graphics when scrolling
PLX                    ;\
INX                    ;} Increment number of times screen has scrolled
STX $0925  ;[$7E:0925] ;/
CPX #$0020             ;\ (32d Bytes) I changed this value from 40 to 20 for Fast Doors
BNE $02                ;} If number of times screen has scrolled = 20h:
SEC                    ;  Return carry set
RTS                    ;
CLC                    ;  Else: return carry clear
RTS                    ;


;-----------------------------------------------------------


; 7520 $80AE7E Door transition - right Modified - Fast Doors

org !7520

LDX $0925  ;[$7E:0925] ;\ Save number of times screen has scrolled
PHX                    ;} Save door transition frame counter
LDA $0AF8  ;[$7E:0AF8] ;\
CLC                    ;|
ADC $092B  ;[$7E:092B] ;} Samus X subposition += [Samus' subspeed during door transition]
STA $0AF8  ;[$7E:0AF8] ;/
LDA $0AF6  ;[$7E:0AF6] ;\
ADC $092D  ;[$7E:092D] ;} Samus X position += [Samus' speed during door transition]
STA $0AF6  ;[$7E:0AF6] ;/
STA $0B10  ;[$7E:0B10] ;  Samus previous X position = [Samus X position]
LDA $0911  ;[$7E:0911] ;\
CLC                    ;|
ADC #$0008             ;} Layer 1 X position += 8, I changed the value from 4 to 8 for Fast Doors
STA $0911  ;[$7E:0911] ;/
LDA $0917  ;[$7E:0917] ;\
CLC                    ;|
ADC #$0008             ;} Layer 2 X position += 8, I changed the value from 4 to 8 for Fast Doors
STA $0917  ;[$7E:0917] ;
CPX #$0015             ;  Decimal values 20,18,16,14,12,10,8,6,4,and,2, execute $80F400: Update BG graphics when scrolling Modified Right Black Blocks
BPL $0F                ;  Branch if Plus or Equal (15d Bytes)
CPX #$0002             ;  Decimal values 20,18,16,14,12,10,8,6,4,and,2, execute $80F400: Update BG graphics when scrolling Modified Right Black Blocks
BMI $0A                ;  Branch if Minus (10d Bytes)
LDA $0925  ;[$7E:0925] ;  number of times screen has scrolled
LSR A                  ;  Divide by 2, even number C-carry clear, non number C-carry set 
BCS $04                ;  Branch if Carry Set to 22 A0 A3 80 JSL $80A3A0 Calculate BG scrolls and update BG graphics when scrolling Original  
JSL $800000+!7400 ;[$80:F400];  $80F400: Update BG graphics when scrolling Modified Right
JSL $80A3A0 ;[$80:A3A0];  Calculate BG positions and update BG graphics when scrolling
PLX                    ;\
INX                    ;} Increment door transition frame counter
STX $0925  ;[$7E:0925] ;/
CPX #$0020             ;\ (32d Bytes) I changed the value from 40 to 20 for Fast Doors
BNE $0C                ;} If [door transition frame counter] = 20h:
LDX #$0000             ;
STX $0925  ;[$7E:0925] ; 
JSL $800000+!7400 ;[$80:F400];  $80F400: Update BG graphics when scrolling Modified Right
SEC                    ;  Return carry set
RTS                    ;
CLC                    ;  Else: return carry clear
RTS                    ;


;-----------------------------------------------------------


; 75A0 $80A3DF Update BG graphics when scrolling Modified Scrolling left & right, saves up & down

org !75A0

; Modified to load Black Blocks outside of the level in the left and right 16:9 area

REP #$20               ;
JSR $A4BB  ;[$80:A4BB] ;  $80A4BB Calculate BG scroll and layer position blocks
LDX #$FFFB             ;  If you scrolling to the left the value will be -5
LDA $08F7  ;[$7E:08F7] ;\ How many blocks X Layer 1 is scrolled
CMP $08FF  ;[$7E:08FF] ;} If [layer 1 block X] = [previous layer 1 block X]: go to BRANCH_LAYER_1_HORIZONTAL_END
BEQ $64                ;/
STA $08FF  ;[$7E:08FF] ;  Previous layer 1 block X = [layer 1 block X]
BMI $03                ;\
LDX #$0015             ;} If scrolled rightwards: X = 15h (21d), else X = FFFB (-5) 16 columns of tiles or 256 x 256 pixels are visible on the 4:3 screen
TXA                    ;\
CLC                    ;|
ADC $08F7  ;[$7E:08F7] ;} Blocks to update X block = [layer 1 block X] + [X]
STA $0990  ;[$7E:0990] ;/
TXA                    ;\
CLC                    ;|
ADC $0907  ;[$7E:0907] ;} VRAM blocks to update X block = [BG1 block X] + [X]
STA $0994  ;[$7E:0994] ;/
LDA $08F9  ;[$7E:08F9] ;\
STA $0992  ;[$7E:0992] ;} Blocks to update Y block = [layer 1 block Y]
LDA $0909  ;[$7E:0909] ;\
STA $0996  ;[$7E:0996] ;} VRAM blocks to update Y block = [BG1 block Y]
LDA $7EFFF6 ;[$7E:FFF6];  $7EFFF6 always has a value of zero, These 5 lines of code can be reused for something else
BEQ $07                ;  Branch if Equal to 0 to E0 15 00    CPX #$0015             ;\ Compare X with #$0015 (Always Branch)
CMP #$0001             ;  Compare A with #$0001 This instruction and the following 2 below are never executed because it always has a value of zero
BEQ $0F                ;  Branch if Equal to 20 D6 A9    JSR $A9D6  [$80:A9D6]  ; Update background data column Original (Never executed)
BRA $12                ;  Branch Always   to 20 05 F2    JSR $F205  [$80:F205]  ; Update level data column  Modified to load Black Blocks (Never executed)
CPX #$0015             ;  Compare X with #$0015 (LDX #$FFFB scrolled left, LDX #$0015 scrolled rightwards)
BEQ $12                ; 
LDA $08F7  ;[$7E:08F7] ;  How many blocks X Layer 1 is scrolled
CMP #$0005             ;  5 is the minimum value to the left, so that normal tiles are loaded within the level,
BMI $05                ;  if it is less than 5, then that tile column is already out of level and black blocks must be loaded
JSR $A9DB  ;[$80:A9DB] ;  Update level data column Original
BRA $1D                ;
JSR !7205  ;[$80:F205] ;  Update level data column  Modified to load Black Blocks
BRA $18                ;
LDA $07A8  ;[$7E:07A8] ;  $7E:07A8 Stores width of the room in pixels (100h,200h,300h,400h or more)
LSR A                  ;  Divide by 2
LSR A                  ;  Divide by 2
LSR A                  ;  Divide by 2
LSR A                  ;  Divide by 2
SEC                    ; 
SBC #$0016             ;  Substrat #$0016h (0022d)
CMP $08F7  ;[$7E:08F7] ;  Compare A with tile of the room,   Position of block X Layer 1
BMI $05                ; 
JSR $A9DB  ;[$80:A9DB] ;  Update level data column Original
BRA $03                ;
JSR !7205  ;[$80:F205] ;  Update level data column Modified to load Black Blocks
LDA $091B  ;[$7E:091B] ;\
LSR A                  ;} If not using custom layer 2: go to BRANCH_LAYER_2_HORIZONTAL_END
BCS $6B                ;/
LDX #$FFFB             ;  If you scrolling to the left the value will be -5
LDA $08FB  ;[$7E:08FB] ;\
CMP $0903  ;[$7E:0903] ;} If [layer 2 block X] = [previous layer 2 block X]: go to BRANCH_LAYER_2_HORIZONTAL_END
BEQ $60                ;/
STA $0903  ;[$7E:0903] ;  Previous layer 2 block X = [layer 2 block X]
BMI $03                ;\
LDX #$0015             ;} If scrolled rightwards: X = 15h (21d), else X = FFFB (-5) 16 columns of tiles or 256 pixels are visible on the 4:3 screen
TXA                    ;\
CLC                    ;|
ADC $08FB  ;[$7E:08FB] ;} Blocks to update X block = [layer 2 block X] + [X]
STA $0990  ;[$7E:0990] ;/
TXA                    ;\
CLC                    ;|
ADC $090B  ;[$7E:090B] ;} VRAM blocks to update X block = [BG2 block X] + [X]
STA $0994  ;[$7E:0994] ;/
LDA $08FD  ;[$7E:08FD] ;\
STA $0992  ;[$7E:0992] ;} Blocks to update Y block = [layer 2 block Y]
LDA $090D  ;[$7E:090D] ;\
STA $0996  ;[$7E:0996] ;} VRAM blocks to update Y block = [BG2 block Y]
LDA $7EFFF6 ;[$7E:FFF6];  $7EFFF6 always has a value of zero, These 5 lines of code can be reused for something else
BEQ $07                ;  Branch if A is Equal to 0 go to CPX #$0015             ;  Compare X with #$0015 (Always Branch)
CMP #$0001             ;  Compare Accumulator with #$0001 (Never executed)
BEQ $0F                ;  Branch if Equal to 20 D6 A9    JSR $A9D6  [$80:A9D6]  ; Update background data column Original (Never executed)
BRA $12                ;  Branch Always   to 20 05 F2    JSR $F205  [$80:F205]  ; Update level data column  Modified to load Black Blocks (Never executed)
CPX #$0015             ;  Compare X with #$0015
BEQ $12                ; 
LDA $08F7  ;[$7E:08F7] ;  How many blocks X Layer 1 is scrolled
CMP #$0005             ; 
BMI $05                ; 
JSR $A9D6  ;[$80:A9D6] ;  Update background data column Original
BRA $1D                ;
JSR !7200  ;[$80:F200] ;  Update background data column Modified to load Black Blocks
BRA $18                ;
LDA $07A8  ;[$7E:07A8] ;  $7E:07A8 Stores width of the room in pixels (100h,200h,300h,400h or more)
LSR A                  ;  Divide by 2
LSR A                  ;  Divide by 2
LSR A                  ;  Divide by 2
LSR A                  ;  Divide by 2
SEC                    ; 
SBC #$0016             ;  Substrat #$0016h (0022d)
CMP $08F7  ;[$7E:08F7] ;  Compare A with tile of the room,   Position of block X Layer 1
BMI $05                ; 
JSR $A9D6  ;[$80:A9D6] ;  Update background data column Original
BRA $03                ;
JSR !7200  ;[$80:F200] ;  Update background data column Modified to load Black Blocks
LDX #$0001             ;
TXA                    ;  Transfer X Register to Accumulator
STA $7EFFF2 ;[$7E:FFF2];  Store Address Absolute Long
LDA $08F9  ;[$7E:08F9] ;\
CMP $0901  ;[$7E:0901] ;} If [layer 1 block Y] = [previous layer 1 block Y]: go to BRANCH_LAYER_1_VERTICAL_END
BEQ $2C                ;/
STA $0901  ;[$7E:0901] ;  Previous layer 1 block Y = [layer 1 block Y]
BMI $08                ;\
LDX #$000F             ;} If scrolled up: X = Fh, else X = 1
TXA                    ;  Transfer X Register to Accumulator
STA $7EFFF2 ;[$7E:FFF2];  Store Address Absolute Long
TXA                    ;\
CLC                    ;|
ADC $08F9  ;[$7E:08F9] ;} Blocks to update Y block = [layer 1 block Y] + [X]
STA $0992  ;[$7E:0992] ;/
TXA                    ;\
CLC                    ;|
ADC $0909  ;[$7E:0909] ;} VRAM blocks to update Y block = [BG1 block Y] + [X]
STA $0996  ;[$7E:0996] ;/
LDA $08F7  ;[$7E:08F7] ;\ How many blocks X Layer 1 is scrolled
STA $0990  ;[$7E:0990] ;} Blocks to update X block = [layer 1 block X]
LDA $0907  ;[$7E:0907] ;\
STA $0994  ;[$7E:0994] ;} VRAM blocks to update X block = [BG1 block X]
JSR $AB75  ;[$80:AB75] ;  Update level data row
LDA $091C  ;[$7E:091C] ;\
LSR A                  ;} If not using custom layer 2: return
BCS $3C                ;/
LDX #$0001             ;
TXA                    ;  Transfer X Register to Accumulator
STA $7EFFF2 ;[$7E:FFF2];  Store Address Absolute Long
LDA $08FD  ;[$7E:08FD] ;\
CMP $0905  ;[$7E:0905] ;} If [layer 2 block Y] = [previous layer 2 block Y]: return
BEQ $2C                ;/
STA $0905  ;[$7E:0905] ;  Previous layer 2 block Y = [layer 2 block Y]
BMI $08                ;\
LDX #$000F             ;} If scrolled up: X = Fh, else X = 1
TXA                    ;  Transfer X Register to Accumulator
STA $7EFFF2 ;[$7E:FFF2];  Store Address Absolute Long
TXA                    ;\
CLC                    ;|
ADC $08FD  ;[$7E:08FD] ;} Blocks to update Y block = [layer 2 block Y] + [X]
STA $0992  ;[$7E:0992] ;/
TXA                    ;\
CLC                    ;|
ADC $090D  ;[$7E:090D] ;} VRAM blocks to update Y block = [BG2 block Y] + [X]
STA $0996  ;[$7E:0996] ;/
LDA $08FB  ;[$7E:08FB] ;\
STA $0990  ;[$7E:0990] ;} Blocks to update X block = [layer 2 block X]
LDA $090B  ;[$7E:090B] ;\
STA $0994  ;[$7E:0994] ;} VRAM blocks to update X block = [BG2 block X]
JSR $AB70  ;[$80:AB70] ;  Update background data row
LDA #$0000             ;  change from 1 to 0 the flag
STA $7EFFF6 ;[$7E:FFF6];  8FF6FF7E  This instruction is only executed once, which means that $7EFFF6 always has a value of zero,
                       ;  These 7 Bytes of code can be reused for something else
PLB                    ;
PLP                    ;
RTL                    ;


;-----------------------------------------------------------


; 7730 First and second round of $80AB78 Update levelbackground data row

org !7730

; Left Screen, values are saved in RAM Memory 7EFFFFD0 and 7EFFFFE0 and later they are recovered to make them DMAs
;;; $AB78: Update level/background data row ;;;

LDA $0783  ;[$7E:0783] ;\
BEQ $01                ;} If mode 7: return
RTS                    ;/
PHP                    ;
REP #$20               ;  Reset Processor Status Bits [Flags affected: all except b per operand] C2 Immediate
                       ;  It is necessary for the following additions to work
TXA                    ;  Transfer X Register to Accumulator
STA $7EFFF4 ;[$7E:FFF4];  Store Address Absolute Long
LDA $7EFFF8 ;[$7E:FFF8];  Load the flag for row zero (values: scroll in row 0 flag=0001, scroll in other row flag=0000) Up or Down only
BNE $14                ;  Branch if number of times screen has scrolled is not 0 (if the flag is 0001) BNE = Not Equal to 0
LDA $0990              ;\
CLC                    ;} Position of the camera, we will move it to the Left Screen
ADC #$FFFB             ;|
STA $0990              ;/ The camera was moved 5 columns of tiles to the Left
LDA $0994              ;\
CLC                    ;} Position of the camera, we will move it to the Left Screen
ADC #$FFFB             ;|
STA $0994              ;/ The camera was moved 5 columns of tiles to the Left
SEP #$20               ;\
LDA $07A5  ;[$7E:07A5] ;|
STA $4202  ;[$7E:4202] ;|
LDA $0992  ;[$7E:0992] ;|
STA $4203  ;[$7E:4203] ;|
PHB                    ;|
REP #$30               ;|
LDA $0990  ;[$7E:0990] ;|
CLC                    ;|
ADC $4216  ;[$7E:4216] ;|
ASL A                  ;} $36 = address of blocks to update (in bank $7F)
CLC                    ;|
ADC #$0002             ;| 690200
TXY                    ;|
BEQ $04                ;|
CLC                    ;|
ADC #$9600             ;|
STA $36    ;[$7E:0036] ;|
LDA #$007F             ;|
STA $38    ;[$7E:0038] ;/
LDA $0994  ;[$7E:0994] ;\
AND #$000F             ;} $0933 = [VRAM blocks to update X block] & Fh
STA $0933  ;[$7E:0933] ;/
LDA #$0010             ;\
SEC                    ;|
SBC $0933  ;[$7E:0933] ;|
ASL A                  ;} Unwrapped tilemap VRAM update size = (10h - ([VRAM blocks to update X block] & 0Fh)) * 4
ASL A                  ;|
STA $0964,x;[$7E:0964] ;/ 9D6409
CPX #$0000             ;  Compare Index Register X with Memory #const Immediate
BNE $06                ;  Branch if Not Equal
STA $7EFFD0 ;[$7E:FFD0];  Store Address Absolute Long
BRA $04                ;  Branch Always
STA $7EFFE0 ;[$7E:FFE0];  Store Address Absolute Long
LDA $0933  ;[$7E:0933] ;\
INC A                  ;|
ASL A                  ;} Wrapped tilemap VRAM update size = (([VRAM blocks to update X block] & 0Fh) + 1) * 4
ASL A                  ;|
STA $0966,x;[$7E:0966] ;/ 9D6609
CPX #$0000             ;  Compare Index Register X with Memory #const Immediate
BNE $06                ;  Branch if Not Equal
STA $7EFFD2 ;[$7E:FFD2];  Store Address Absolute Long
BRA $04                ;  Branch Always
STA $7EFFE2 ;[$7E:FFE2];  Store Address Absolute Long
SEP #$20               ;\
LDA $0996  ;[$7E:0996] ;|
AND #$0F               ;|
STA $4202  ;[$7E:4202] ;|
LDA #$40               ;|
STA $4203  ;[$7E:4203] ;|
REP #$20               ;|
LDA $0994  ;[$7E:0994] ;} VRAM offset of blocks to update = (([VRAM blocks to update Y block] & Fh) * 20h + [X block of VRAM blocks to update]
                       ;| & 1Fh) * 2
AND #$001F             ;|
STA $0935  ;[$7E:0935] ;|
ASL A                  ;|
CLC                    ;|
ADC $4216  ;[$7E:4216] ;|
STA $0933  ;[$7E:0933] ;/
LDA #$5400             ;\
STA $0937  ;[$7E:0937] ;} Base address of wrapped VRAM tilemap screen = VRAM map 2nd screen base address
LDA #$5000             ;  A = VRAM map 1st screen base address
LDY $0935  ;[$7E:0935] ;\
CPY #$0010             ;} If [X block of VRAM blocks to update] & 1Fh >= 10h:
BCC $09                ;/
LDA #$5000             ;\
STA $0937  ;[$7E:0937] ;} Base address of wrapped VRAM tilemap screen = VRAM map 1st screen base address
LDA #$53E0             ;  A = VRAM map 2nd screen base address (devs opted to subtract 20h here, rather than take $0935 & Fh before summing)
TXY                    ;\
BEQ $04                ;} If background update:
SEC                    ;\
SBC $098E  ;[$7E:098E] ;} A -= [size of BG2]
CLC                    ;\
ADC $0933  ;[$7E:0933] ;} Unwrapped tilemap VRAM update destination = [A] + [VRAM offset of blocks to update]
STA $0968,x;[$7E:0968] ;/ 9D6809
CPX #$0000             ;  Compare Index Register X with Memory #const Immediate
BNE $06                ;  Branch if Not Equal
STA $7EFFD4 ;[$7E:FFD4];  Store Address Absolute Long
BRA $04                ;  Branch Always
STA $7EFFE4 ;[$7E:FFE4];  Store Address Absolute Long
LDA $0937  ;[$7E:0937] ;  A = [base address of wrapped VRAM tilemap screen]
TXY                    ;\
BEQ $04                ;} If background update:
SEC                    ;\
SBC $098E  ;[$7E:098E] ;} A -= [size of BG2]
CLC                    ;\
ADC $4216  ;[$7E:4216] ;} Wrapped tilemap VRAM update destination = [A] + (VRAM blocks to update Y position & 0Fh) * 20h * 2
STA $096A,x;[$7E:096A] ;/ 9D6A09
CPX #$0000             ;  Compare Index Register X with Memory #const Immediate
BNE $06                ;  Branch if Not Equal
STA $7EFFD6 ;[$7E:FFD6];  Store Address Absolute Long
BRA $04                ;  Branch Always
STA $7EFFE6 ;[$7E:FFE6];  Store Address Absolute Long
LDA #$C948             ;\
LDY #$0000             ;|
CPX #$0000             ;|
BEQ $06                ;} New tilemap source address = $C948 if level update, else $CA50 (background update)
LDA #$CA50             ;|
LDY #$0108             ;/
CLC                    ;\
ADC $0964,x;[$7E:0964] ;} Wrapped tilemap VRAM update top halves source = new tilemap source address + [unwrapped tilemap VRAM update size]
STA $096C,x;[$7E:096C] ;/ 9D6C09
CPX #$0000             ;  Compare Index Register X with Memory #const Immediate
BNE $06                ;  Branch if Not Equal
STA $7EFFD8 ;[$7E:FFD8];  Store Address Absolute Long
BRA $04                ;  Branch Always
STA $7EFFE8 ;[$7E:FFE8];  Store Address Absolute Long
CLC                    ;\
ADC #$0044             ;} Wrapped tilemap VRAM update bottom halves source = [wrapped tilemap VRAM update top halves source] + 44h
STA $096E,x;[$7E:096E] ;/ 9D6E09
CPX #$0000             ;  Compare Index Register X with Memory #const Immediate
BNE $06                ;  Branch if Not Equal
STA $7EFFDA ;[$7E:FFDA];  Store Address Absolute Long
BRA $04                ;  Branch Always
STA $7EFFEA ;[$7E:FFEA];  Store Address Absolute Long
STY $0937  ;[$7E:0937] ;  $0937 = index of VRAM tilemap source data
SEP #$20               ;\
LDA #$7E               ;|
PHA                    ;} DB = $7E
PLB                    ;|
REP #$20               ;/
PHX                    ;  Save X parameter
LDY #$0000             ;  Y = 0 (nth row offset for block to update)
LDA #$0011             ;\
STA $0939  ;[$7E:0939] ;} $0939 = 11h (counter for following loop)
        ; LOOP 0 After executing the LOOP below once, return here to check if the next block is outside of the level area and 
        ; we must load a Black Block, if so we do BRAnch or Jump to LOOP 2
LDA $08F7  ;[$7E:08F7] ;\ $7E:087F Stores tile of the room scrolling, 0 limit of the left   How many blocks X Layer 1 is scrolled
CLC                    ;|
ADC #$000C             ;| Add #$000Ch (0012d) to Accumulator
CMP $0939  ;[$7E:0939] ;} Compare A with $0939 = 11h (counter for following loop)
BMI $02                ;\
BRA $41                ;  [BRA +65d] ;} Branch to LOOP 1
        ; LOOP 2 B7368D3B09
LDA [$36],y;[$7F:0202] ;\
STA $093B  ;[$7E:093B] ;} $093B = block to update (in $7F)
PHY                    ;  Push Y
LDA $079F  ;[$7E:079F] ;  Region Number  0000 Crateria, 0001 Brinstar, 0002 Norfair, 0003 Wrecked Ship, 0004 Maridia, 0005 Tourian, 0006 Ceres Station
CMP #$0006             ;  Compare A with #$0006  0006 Ceres Station
BNE $10                ;  BNE to AC 37 09    LDY $0937  [$7E:0937]  ; Y = [index of VRAM tilemap source data]
LDA #$2803             ;  (2803 air block in Ceres Station, 00DC air block in Ridley's Room of Ceres Station, 2323 black block)    
LDY $079B  ;[$7E:079B] ;  Number of Room in current area 
CPY #$E0B5             ;  Compare A with #$E0B5 (Ridley's Room in Ceres Station)
BNE $08                ; 
LDA #$00DC             ;  (2803 air block in Ceres Station, 00DC air block in Ridley's Room of Ceres Station, 2323 black block)
BRA $03                ;
LDA #$2323             ;  (0338, 2323, 0A21) change A9 00 00 to A9 23 23
LDY $0937  ;[$7E:0937] ;  Y = [index of VRAM tilemap source data]
STA $C948,y;[$7E:C948] ;} Store top-left VRAM tile of the block
STA $C94A,y;[$7E:C94A] ;} Store top-right VRAM tile of the block
STA $C98C,y;[$7E:C98C] ;} Store bottom-left VRAM tile of the block
STA $C98E,y;[$7E:C98E] ;} Store bottom-right VRAM tile of the block
INY                    ;\
INY                    ;|
INY                    ;} Index of VRAM tilemap source data += 4
INY                    ;|
STY $0937  ;[$7E:0937] ;/
PLY                    ;\
INY                    ;} Y = next block offset
INY                    ;/
DEC $0939  ;[$7E:0939] ;  Decrement $0939
BEQ $02                ;\
BRA $B1                ;  [BRA -79d]  ;} If [$0939] != 0: go to LOOP 0
        ; LOOP 1
LDA [$36],y;[$7F:0202] ;\
STA $093B  ;[$7E:093B] ;} $093B = block to update (in $7F)
AND #$03FF             ;\
ASL A                  ;|
ASL A                  ;} X = tile number of block * 8 (offset of tile table at $7E:A000)
ASL A                  ;|
TAX                    ;/
PHY                    ;  Push Y
LDY $0937  ;[$7E:0937] ;  Y = [index of VRAM tilemap source data]
LDA $093B  ;[$7E:093B] ;\
AND #$0C00             ;} If block has no flip:
BNE $1B                ;/
LDA $A000,x;[$7E:A158] ;\
STA $C948,y;[$7E:C948] ;} Copy top-left VRAM tile of the block
LDA $A002,x;[$7E:A15A] ;\
STA $C94A,y;[$7E:C94A] ;} Copy top-right VRAM tile of the block
LDA $A004,x;[$7E:A15C] ;\
STA $C98C,y;[$7E:C98C] ;} Copy bottom-left VRAM tile of the block
LDA $A006,x;[$7E:A15E] ;\
STA $C98E,y;[$7E:C98E] ;} Copy bottom-right VRAM tile of the block
BRA $7B                ;  Go to BRANCH_NEXT
NOP                    ;
CMP #$0400             ;\
BNE $26                ;} Else if block has only horizontal flip:
LDA $A002,x;[$7E:A172] ;\
EOR #$4000             ;|
STA $C948,y;[$7E:C954] ;|
LDA $A000,x;[$7E:A170] ;|
EOR #$4000             ;|
STA $C94A,y;[$7E:C956] ;|
LDA $A006,x;[$7E:A176] ;} Copy the four VRAM tiles of the block, with horizontal flip
EOR #$4000             ;|
STA $C98C,y;[$7E:C998] ;|
LDA $A004,x;[$7E:A174] ;|
EOR #$4000             ;|
STA $C98E,y;[$7E:C99A] ;/
BRA $4F                ;  Go to BRANCH_NEXT
CMP #$0800             ;\
BNE $26                ;} Else if block has only vertical flip:
LDA $A004,x;[$7E:A17C] ;\
EOR #$8000             ;|
STA $C948,y;[$7E:C950] ;|
LDA $A006,x;[$7E:A17E] ;|
EOR #$8000             ;|
STA $C94A,y;[$7E:C952] ;|
LDA $A000,x;[$7E:A178] ;} Copy the four VRAM tiles of the block, with vertical flip
EOR #$8000             ;|
STA $C98C,y;[$7E:C994] ;|
LDA $A002,x;[$7E:A17A] ;|
EOR #$8000             ;|
STA $C98E,y;[$7E:C996] ;/
BRA $24                ;  Go to BRANCH_NEXT
LDA $A006,x;[$7E:A00E] ;\ Else: block has horizontal and vertical flips
EOR #$C000             ;|
STA $C948,y;[$7E:C968] ;|
LDA $A004,x;[$7E:A00C] ;|
EOR #$C000             ;|
STA $C94A,y;[$7E:C96A] ;|
LDA $A002,x;[$7E:A00A] ;} Copy the four VRAM tiles of the block, with horizontal and vertical flips
EOR #$C000             ;|
STA $C98C,y;[$7E:C9AC] ;|
LDA $A000,x;[$7E:A008] ;|
EOR #$C000             ;|
STA $C98E,y;[$7E:C9AE] ;/
        ; BRANCH_NEXT
INY                    ;\
INY                    ;|
INY                    ;}  Index of VRAM tilemap source data += 4
INY                    ;|
STY $0937  ;[$7E:0937] ;/
PLY                    ;\
INY                    ;}  Y = next block offset
INY                    ;/
DEC $0939  ;[$7E:0939] ;  Decrement $0939
BEQ $03                ;\
BRL $FF41              ;  [BRL -206d] ;} If [$0939] != 0: go to LOOP 0  FF41h es igual to -191
PLX                    ;  Restore X parameter
INC $0970,x;[$7E:0970] ;  FE7009 Set flag to perform VRAM update DMAs
LDA $0970  ;[$7E:0970] ;  Load Absolute Address
CPX #$0000             ;  Compare Index Register X with Memory #const Immediate
BNE $06                ;  Branch if Not Equal
STA $7EFFDC ;[$7E:FFDC];  Store Address Absolute Long
BRA $04                ;  Branch Always
STA $7EFFEC ;[$7E:FFEC];  Store Address Absolute Long
PLB                    ;
PLP                    ;
;60          RTS  I removed the Return because it's necessary to continue with round 2 of $80AB78: Update level/background data row


; Right Screen, from here, the code is executed normally until DMAs
; 7730 First and second round of $80AB78 Update levelbackground data row
;;; $AB78: Update level/background data row ;;;

LDA $0783  ;[$7E:0783] ;\
BEQ $01                ;} If mode 7: return
RTS                    ;/
PHP                    ;
REP #$20               ;  Reset Processor Status Bits [Flags affected: all except b per operand] C2 Immediate
                       ;  m=0, x=1, A/M=16-bit, X/Y=8-bit, Instruction=REP #$20
LDA $7EFFF4 ;[$7E:FFF4];  Load Address Absolute Long
TAX                    ;  Transfer Accumulator to Register X
LDA $7EFFF8 ;[$7E:FFF8];  Load the flag for row zero (values: scroll in row 0 flag=0001, scroll in other row flag=0000) Up or Down only
BEQ $09                ;  Branch if number of times screen has scrolled is more than zero (if the flag is 0000)
LDA #$0000             ;\
STA $7EFFF8 ;[$7E:FFF8];} clear flag in the row 0, when elevators and doors go up and down
BRA $16                ;
LDA $0990              ;\
CLC                    ;} Position of the camera, we will move it to the Right Screen
ADC #$000A             ;|
STA $0990              ;/ The camera was moved 10 columns of tiles to the right
LDA $0994              ;\
CLC                    ;} Position of the camera, we will move it to the Right Screen
ADC #$000A             ;|
STA $0994              ;/ The camera was moved 10 columns of tiles to the right
BRA $14                ;
LDA $0990              ;\
CLC                    ;|
ADC #$0010             ;|
STA $0990              ;} We move the camera for the row 0 of tiles when we go up or down, a whole screen to the right
LDA $0994              ;| This "fixes" the garbage row of tiles of certain elevators and doors going up and down
CLC                    ;|
ADC #$0010             ;|
STA $0994              ;/
SEP #$20               ;\
LDA $07A5  ;[$7E:07A5] ;|
STA $4202  ;[$7E:4202] ;|
LDA $0992  ;[$7E:0992] ;|
STA $4203  ;[$7E:4203] ;|
PHB                    ;|
REP #$30               ;|
LDA $0990  ;[$7E:0990] ;|
CLC                    ;|
ADC $4216  ;[$7E:4216] ;|
ASL A                  ;} $36 = address of blocks to update (in bank $7F)
CLC                    ;|
ADC #$0002             ;|
TXY                    ;|
BEQ $04                ;|
CLC                    ;|
ADC #$9600             ;|
STA $36    ;[$7E:0036] ;|
LDA #$007F             ;|
STA $38    ;[$7E:0038] ;/
LDA $0994  ;[$7E:0994] ;\
AND #$000F             ;} $0933 = [VRAM blocks to update X block] & Fh
STA $0933  ;[$7E:0933] ;/
LDA #$0010             ;\
SEC                    ;|
SBC $0933  ;[$7E:0933] ;|
ASL A                  ;} Unwrapped tilemap VRAM update size = (10h - ([VRAM blocks to update X block] & 0Fh)) * 4
ASL A                  ;|
STA $0964,x;[$7E:0964] ;/
LDA $0933  ;[$7E:0933] ;\
INC A                  ;|
ASL A                  ;} Wrapped tilemap VRAM update size = (([VRAM blocks to update X block] & 0Fh) + 1) * 4
ASL A                  ;|
STA $0966,x;[$7E:0966] ;/
SEP #$20               ;\
LDA $0996  ;[$7E:0996] ;|
AND #$0F               ;|
STA $4202  ;[$7E:4202] ;|
LDA #$40               ;|
STA $4203  ;[$7E:4203] ;|
REP #$20               ;|
LDA $0994  ;[$7E:0994] ;} VRAM offset of blocks to update = (([VRAM blocks to update Y block] & Fh) * 20h + [X block of VRAM blocks to update]
                       ;| & 1Fh) * 2
AND #$001F             ;|
STA $0935  ;[$7E:0935] ;|
ASL A                  ;|
CLC                    ;|
ADC $4216  ;[$7E:4216] ;|
STA $0933  ;[$7E:0933] ;/
LDA #$5400             ;\
STA $0937  ;[$7E:0937] ;} Base address of wrapped VRAM tilemap screen = VRAM map 2nd screen base address
LDA #$5000             ; A = VRAM map 1st screen base address
LDY $0935  ;[$7E:0935] ;\
CPY #$0010             ;} If [X block of VRAM blocks to update] & 1Fh >= 10h:
BCC $09                ;/
LDA #$5000             ;\
STA $0937  ;[$7E:0937] ;} Base address of wrapped VRAM tilemap screen = VRAM map 1st screen base address
LDA #$53E0             ; A = VRAM map 2nd screen base address (devs opted to subtract 20h here, rather than take $0935 & Fh before summing)
TXY                    ;\
BEQ $04                ;} If background update:
SEC                    ;\
SBC $098E  ;[$7E:098E] ;} A -= [size of BG2]
CLC                    ;\
ADC $0933  ;[$7E:0933] ;} Unwrapped tilemap VRAM update destination = [A] + [VRAM offset of blocks to update]
STA $0968,x;[$7E:0968] ;/
LDA $0937  ;[$7E:0937] ; A = [base address of wrapped VRAM tilemap screen]
TXY                    ;\
BEQ $04                ;} If background update:
SEC                    ;\
SBC $098E  ;[$7E:098E] ;} A -= [size of BG2]
CLC                    ;\
ADC $4216  ;[$7E:4216] ;} Wrapped tilemap VRAM update destination = [A] + (VRAM blocks to update Y position & 0Fh) * 20h * 2
STA $096A,x;[$7E:096A] ;/
LDA #$CB48             ;\ LDA #$C948  changed to  LDA #$CB48
LDY #$0000             ;|
CPX #$0000             ;|
BEQ $06                ;} New tilemap source address = $CB48 if level update, else $CC50 (background update)
LDA #$CC50             ;| LDA #$CA50  changed to  LDA #$CC50
LDY #$0108             ;/
CLC                    ;\
ADC $0964,x;[$7E:0964] ;} Wrapped tilemap VRAM update top halves source = new tilemap source address + [unwrapped tilemap VRAM update size]
STA $096C,x;[$7E:096C] ;/
CLC                    ;\
ADC #$0044             ;} Wrapped tilemap VRAM update bottom halves source = [wrapped tilemap VRAM update top halves source] + 44h
STA $096E,x;[$7E:096E] ;/
STY $0937  ;[$7E:0937] ; $0937 = index of VRAM tilemap source data
SEP #$20               ;\
LDA #$7E               ;|
PHA                    ;} DB = $7E
PLB                    ;|
REP #$20               ;/
PHX                    ; Save X parameter
LDY #$0000             ; Y = 0 (nth row offset for block to update)
LDA #$0011             ;\
STA $0939  ;[$7E:0939] ;} $0939 = 11h (counter for following loop)
        ; LOOP 0 After executing the LOOP below once, return here to check if the next block is outside of the level area and 
        ; we must load a Black Block, if so we do BRAnch or Jump to LOOP 2
LDA $07A8  ;[$7E:07A8] ;  $7E:07A8 Stores width of the room in pixels (100h,200h,300h,400h or more)
LSR A                  ;  Divide by 2
LSR A                  ;  Divide by 2
LSR A                  ;  Divide by 2
LSR A                  ;  Divide by 2
SEC                    ; 
SBC #$0017             ;  Substrat #$0017h (0023d)
CLC                    ; 
ADC $0939  ;[$7E:0939] ;  Add $7E:0939 to Accumulator, values between 17 and 1
CMP $08F7  ;[$7E:08F7] ;  Compare A with tile of the room,   Position of block X Layer 1
BPL $03                ; 
BRL $00C1              ;  Branch Long to LOOP 2
        ; LOOP 1
LDA [$36],y;[$7F:0202] ;\
STA $093B  ;[$7E:093B] ;} $093B = block to update (in $7F)
AND #$03FF             ;\
ASL A                  ;|
ASL A                  ;} X = tile number of block * 8 (offset of tile table at $7E:A000)
ASL A                  ;|
TAX                    ;/
PHY                    ;  Push Y
LDY $0937  ;[$7E:0937] ;  Y = [index of VRAM tilemap source data]
LDA $093B  ;[$7E:093B] ;\
AND #$0C00             ;} If block has no flip:
BNE $1B                ;/
LDA $A000,x;[$7E:A158] ;\
STA $CB48,y;[$7E:CB48] ;} STA $C948,y  changed to  STA $CB48,y  Copy top-left VRAM tile of the block
LDA $A002,x;[$7E:A15A] ;\
STA $CB4A,y;[$7E:CB4A] ;} STA $C94A,y  changed to  STA $CB4A,y  Copy top-right VRAM tile of the block
LDA $A004,x;[$7E:A15C] ;\
STA $CB8C,y;[$7E:CB8C] ;} STA $C98C,y  changed to  STA $CB8C,y  Copy bottom-left VRAM tile of the block
LDA $A006,x;[$7E:A15E] ;\
STA $CB8E,y;[$7E:CB8E] ;} STA $C98E,y  changed to  STA $CB8E,y  Copy bottom-right VRAM tile of the block
BRA $7B                ; Go to BRANCH_NEXT
NOP                    ;
CMP #$0400             ;\
BNE $26                ;} Else if block has only horizontal flip:
LDA $A002,x;[$7E:A172] ;\
EOR #$4000             ;|
STA $CB48,y;[$7E:CB54] ;| STA $C948,y  changed to  STA $CB48,y
LDA $A000,x;[$7E:A170] ;|
EOR #$4000             ;|
STA $CB4A,y;[$7E:CB56] ;| STA $C94A,y  changed to  STA $CB4A,y
LDA $A006,x;[$7E:A176] ;} Copy the four VRAM tiles of the block, with horizontal flip
EOR #$4000             ;|
STA $CB8C,y;[$7E:CB98] ;| STA $C98C,y  changed to  STA $CB8C,y
LDA $A004,x;[$7E:A174] ;|
EOR #$4000             ;|
STA $CB8E,y;[$7E:CB9A] ;/ STA $C98E,y  changed to  STA $CB8E,y
BRA $4F                ;  Go to BRANCH_NEXT
CMP #$0800             ;\
BNE $26                ;} Else if block has only vertical flip:
LDA $A004,x;[$7E:A17C] ;\
EOR #$8000             ;|
STA $CB48,y;[$7E:CB50] ;| STA $C948,y  changed to  STA $CB48,y
LDA $A006,x;[$7E:A17E] ;|
EOR #$8000             ;|
STA $CB4A,y;[$7E:CB52] ;| STA $C94A,y  changed to  STA $CB4A,y
LDA $A000,x;[$7E:A178] ;} Copy the four VRAM tiles of the block, with vertical flip
EOR #$8000             ;|
STA $CB8C,y;[$7E:CB94] ;| STA $C98C,y  changed to  STA $CB8C,y
LDA $A002,x;[$7E:A17A] ;|
EOR #$8000             ;|
STA $CB8E,y;[$7E:CB96] ;/ STA $C98E,y  changed to  STA $CB8E,y
BRA $24                ;  Go to BRANCH_NEXT
LDA $A006,x;[$7E:A00E] ;\ Else: block has horizontal and vertical flips
EOR #$C000             ;|
STA $CB48,y;[$7E:CB68] ;| STA $C948,y  changed to  STA $CB48,y
LDA $A004,x;[$7E:A00C] ;|
EOR #$C000             ;|
STA $CB4A,y;[$7E:CB6A] ;| STA $C94A,y  changed to  STA $CB4A,y
LDA $A002,x;[$7E:A00A] ;} Copy the four VRAM tiles of the block, with horizontal and vertical flips
EOR #$C000             ;|
STA $CB8C,y;[$7E:CBAC] ;| STA $C98C,y  changed to  STA $CB8C,y
LDA $A000,x;[$7E:A008] ;|
EOR #$C000             ;|
STA $CB8E,y;[$7E:CBAE] ;/ STA $C98E,y  changed to  STA $CB8E,y
        ; BRANCH_NEXT
INY                    ;\
INY                    ;|
INY                    ;} Index of VRAM tilemap source data += 4
INY                    ;|
STY $0937  ;[$7E:0937] ;/
PLY                    ;\
INY                    ;} Y = next block offset
INY                    ;/
DEC $0939  ;[$7E:0939] ;  Decrement $0939
BEQ $03                ; 
BRL $FF2A              ;  If [$0939] != 0: go to LOOP FF2Ah es igual to -214
BRA $41                ;  BRAnch to $80:AD16 FA PLX ; Restore X parameter and it comes out of this subroutine [BRA  +65]
        ; LOOP 2 B7368D3B09
LDA [$36],y;[$7F:0202] ;\
STA $093B  ;[$7E:093B] ;} $093B = block to update (in $7F)
PHY                    ;  Push Y
LDA $079F  ;[$7E:079F] ;  Region Number  0000 Crateria, 0001 Brinstar, 0002 Norfair, 0003 Wrecked Ship, 0004 Maridia, 0005 Tourian, 0006 Ceres Station
CMP #$0006             ;  Compare A with #$0006  0006 Ceres Station
BNE $10                ;  BNE to AC 37 09    LDY $0937  [$7E:0937]  ; Y = [index of VRAM tilemap source data]
LDA #$2803             ;  (2803 air block in Ceres Station, 00DC air block in Ridley's Room of Ceres Station, 2323 black block)    
LDY $079B  ;[$7E:079B] ;  Number of Room in current area 
CPY #$E0B5             ;  Compare A with #$E0B5 (Ridley's Room in Ceres Station) 7E079B E0B5 Ceres Ridley's Room
BNE $08                ; 
LDA #$00DC             ;  (2803 air block in Ceres Station, 00DC air block in Ridley's Room of Ceres Station, 2323 black block)
BRA $03                ;
LDA #$2323             ;  (0338, 2323, 0A21) change A9 00 00 to A9 23 23, change A92323 to A90338
LDY $0937  ;[$7E:0937] ;  Y = [index of VRAM tilemap source data]
STA $CB48,y;[$7E:CB48] ;} STA $C948,y  changed to  STA $CB48,y  Store top-left VRAM tile of the block
STA $CB4A,y;[$7E:CB4A] ;} STA $C94A,y  changed to  STA $CB4A,y  Store top-right VRAM tile of the block
STA $CB8C,y;[$7E:CB8C] ;} STA $C98C,y  changed to  STA $CB8C,y  Store bottom-left VRAM tile of the block
STA $CB8E,y;[$7E:CB8E] ;} STA $C98E,y  changed to  STA $CB8E,y  Store bottom-right VRAM tile of the block
        ; BRANCH_NEXT
INY                    ;\
INY                    ;|
INY                    ;} Index of VRAM tilemap source data += 4
INY                    ;|
STY $0937  ;[$7E:0937] ;/
PLY                    ;\
INY                    ;} Y = next block offset
INY                    ;/
DEC $0939  ;[$7E:0939] ;  Decrement $0939
BEQ $02                ;\
BRA $BF                ;} If [$0939] != 0: go to LOOP 2 [BRA  -65]
PLX                    ;  Restore X parameter
INC $0970,x;[$7E:0970] ;  Set flag to perform VRAM update DMAs
PLB                    ;
PLP                    ;
RTS                    ;


;-----------------------------------------------------------


; 7C10 $808DAC Execute vertical scrolling DMAs 2 rounds

org !7C10

; Make DMAs to Right screen

LDA #$80              ;
STA $2115  ;[$7E2115] ;} VRAM address increment mode = 16-bit access
LDA $0970  ;[$7E0970] ;
BEQ $70               ;} If vertical BG1 VRAM update flag
NOP                   ;  Don't Clear vertical BG1 VRAM update flag
NOP                   ;
NOP                   ;
LDY $0968  ;[$7E0968] ;
STY $2116  ;[$7E2116] ;
LDX #$1801            ;
STX $4310  ;[$7E4310] ;
LDX #$CB48            ;  LDX #$C948  changed to  LDX #$CB48
STX $4312  ;[$7E4312] ;
LDA #$7E              ;} DMA [$0964] bytes from $7ECB48 vertical scroll top tilemap to [$0968] VRAM
STA $4314  ;[$7E4314] ;
LDX $0964  ;[$7E0964] ;
STX $4315  ;[$7E4315] ;
LDA #$02              ;
STA $420B  ;[$7E420B] ;
REP #$20              ;
TYA                   ;
ORA #$0020            ;
STA $2116  ;[$7E2116] ;
SEP #$20              ;
STX $4315  ;[$7E4315] ;} DMA [$0964] bytes from $7ECB8C vertical scroll bottom tilemap to [$0968]20h VRAM
LDX #$CB8C            ;  LDX #$C98C  changed to  LDX #$CB8C
STX $4312  ;[$7E4312] ;
LDA #$02              ;
STA $420B  ;[$7E420B] ;
LDX $096C  ;[$7E096C] ;
STX $4312  ;[$7E4312] ;
LDX $0966  ;[$7E0966] ; 
BEQ $27               ;} If [$0966]
STX $4315  ;[$7E4315] ;
LDY $096A  ;[$7E096A] ;} DMA [$0966] bytes from $7E[$096C] to [$096A] VRAM
STY $2116  ;[$7E2116] ;
LDA #$02              ;
STA $420B  ;[$7E420B] ;
REP #$20              ;
TYA                   ;
ORA #$0020            ;
STA $2116  ;[$7E2116] ;
SEP #$20              ;
STX $4315  ;[$7E4315] ;} DMA [$0966] bytes from $7E[$096E] to [$096A]20h VRAM
LDX $096E  ;[$7E096E] ;
STX $4312  ;[$7E4312] ;
LDA #$02              ;
STA $420B  ;[$7E420B] ;
LDA $098C  ;[$7E098C] ;
BEQ $76               ;} If [$098C]
NOP                   ;  Don't Clear $098C
NOP                   ;
NOP                   ;
LDY $0984  ;[$7E0984] ;
STY $2116  ;[$7E2116] ;
LDX #$1801            ;
STX $4310  ;[$7E4310] ;
LDX #$CC50            ;  LDX #$CA50  changed to  LDX #$CC50
STX $4312  ;[$7E4312] ;
LDA #$7E              ;} DMA [$0980] bytes from $7ECC50 to [$0984] VRAM
STA $4314  ;[$7E4314] ;
LDX $0980  ;[$7E0980] ;
STX $4315  ;[$7E4315] ;
LDA #$02              ;
STA $420B  ;[$7E420B] ;
REP #$20              ;
TYA                   ;
ORA #$0020            ;
STA $2116  ;[$7E2116] ;
SEP #$20              ;
STX $4315  ;[$7E4315] ;} DMA [$0980] bytes from $7ECC94 to [$0984]20h VRAM
LDX #$CC94            ;  LDX #$CA94  changed to  LDX #$CC94
STX $4312  ;[$7E4312] ;
LDA #$02              ;
STA $420B  ;[$7E420B] ;
LDX $0988  ;[$7E0988] ;
STX $4312  ;[$7E4312] ;
LDX $0982  ;[$7E0982] ; 
BEQ $2D               ;} If [$0982] = 0 return
STX $4315  ;[$7E4315] ;
LDY $0986  ;[$7E0986] ;} DMA [$0982] bytes from $7E[$0988] to [$0986] VRAM
STY $2116  ;[$7E2116] ;
LDA #$02              ;
STA $420B  ;[$7E420B] ;
SEP #$02              ;
BEQ $02               ;} Strange unconditional jump o_O
BRA $FE               ;  Crash
REP #$20              ;
TYA                   ;
ORA #$0020            ;
STA $2116  ;[$7E2116] ;
SEP #$20              ;
STX $4315  ;[$7E4315] ;} DMA [$0982] bytes from $7E[$098A] to [$0986]20h VRAM
LDX $098A  ;[$7E098A] ;
STX $4312  ;[$7E4312] ;
LDA #$02              ;
STA $420B  ;[$7E420B] ;
;RTS                  ;  I removed the Return because we continue with the second round


; Recover Left screen values and make DMAs

REP #$20              ;
LDA $0970 ;[$7E:0970] ;\
BEQ $31               ;} If vertical BG1 VRAM update flag:
LDA $7EFFD0           ;
STA $0964             ;
LDA $7EFFD2           ;
STA $0966             ;
LDA $7EFFD4           ;
STA $0968             ;
LDA $7EFFD6           ;
STA $096A             ;
LDA $7EFFD8           ;
STA $096C             ;
LDA $7EFFDA           ;
STA $096E             ;
NOP                   ;
NOP                   ;
NOP                   ;
NOP                   ;
NOP                   ;
NOP                   ;
NOP                   ;
LDA $098C ;[$7E:098C] ;\
BEQ $31               ;} If [$098C] is EQual to 0 (cero), BRAnch $76 Bytes (118 Bytes dec):
LDA $7EFFE0           ;
STA $0980             ;
LDA $7EFFE2           ;
STA $0982             ;
LDA $7EFFE4           ;
STA $0984             ;
LDA $7EFFE6           ;
STA $0986             ;
LDA $7EFFE8           ;
STA $0988             ;
LDA $7EFFEA           ;
STA $098A             ;
NOP                   ;
NOP                   ;
NOP                   ;
NOP                   ;
NOP                   ;
NOP                   ;
NOP                   ;
SEP #$20              ;
LDA #$80              ;
STA $2115  ;[$7E2115] ;} VRAM address increment mode = 16-bit access
LDA $0970  ;[$7E0970] ;
BEQ $70               ;} If vertical BG1 VRAM update flag
STZ $0970  ;[$7E0970] ; Clear vertical BG1 VRAM update flag
LDY $0968  ;[$7E0968] ;
STY $2116  ;[$7E2116] ;
LDX #$1801            ;
STX $4310  ;[$7E4310] ;
LDX #$C948            ;
STX $4312  ;[$7E4312  ;
LDA #$7E              ;} DMA [$0964] bytes from $7EC948 vertical scroll top tilemap to [$0968] VRAM
STA $4314  ;[$7E4314] ;
LDX $0964  ;[$7E0964] ;
STX $4315  ;[$7E4315] ;
LDA #$02              ;
STA $420B  ;[$7E420B] ;
REP #$20              ;
TYA                   ;
ORA #$0020            ;
STA $2116  ;[$7E2116] ;
SEP #$20              ;
STX $4315  ;[$7E4315] ;} DMA [$0964] bytes from $7EC98C vertical scroll bottom tilemap to [$0968]20h VRAM
LDX #$C98C            ;
STX $4312  ;[$7E4312] ;
LDA #$02              ;
STA $420B  ;[$7E420B] ;
LDX $096C  ;[$7E096C] ;
STX $4312  ;[$7E4312] ;
LDX $0966  ;[$7E0966] ; 
BEQ $27               ;} If [$0966]
STX $4315  ;[$7E4315] ;
LDY $096A  ;[$7E096A] ;} DMA [$0966] bytes from $7E[$096C] to [$096A] VRAM
STY $2116  ;[$7E2116] ;
LDA #$02              ;
STA $420B  ;[$7E420B] ;
REP #$20              ;
TYA                   ;
ORA #$0020            ;
STA $2116  ;[$7E2116] ;
SEP #$20              ;
STX $4315  ;[$7E4315] ;} DMA [$0966] bytes from $7E[$096E] to [$096A]20h VRAM
LDX $096E  ;[$7E096E] ;
STX $4312  ;[$7E4312] ;
LDA #$02              ;
STA $420B  ;[$7E420B] ;
LDA $098C  ;[$7E098C] ;
BEQ $76               ;} If [$098C]
STZ $098C  ;[$7E098C] ;  Clear $098C
LDY $0984  ;[$7E0984] ;
STY $2116  ;[$7E2116] ;
LDX #$1801            ;
STX $4310  ;[$7E4310] ;
LDX #$CA50            ;
STX $4312  ;[$7E4312] ;
LDA #$7E              ;} DMA [$0980] bytes from $7ECA50 to [$0984] VRAM
STA $4314  ;[$7E4314] ;
LDX $0980  ;[$7E0980] ;
STX $4315  ;[$7E4315] ;
LDA #$02              ;
STA $420B  ;[$7E420B] ;
REP #$20              ;
TYA                   ;
ORA #$0020            ;
STA $2116  ;[$7E2116] ;
SEP #$20              ;
STX $4315  ;[$7E4315] ;} DMA [$0980] bytes from $7ECA94 to [$0984]20h VRAM
LDX #$CA94            ;
STX $4312  ;[$7E4312] ;
LDA #$02              ;
STA $420B  ;[$7E420B] ;
LDX $0988  ;[$7E0988] ;
STX $4312  ;[$7E4312  ;
LDX $0982  ;[$7E0982] ; 
BEQ $2D               ;} If [$0982] = 0 return
STX $4315  ;[$7E4315] ;
LDY $0986  ;[$7E0986] ;} DMA [$0982] bytes from $7E[$0988] to [$0986] VRAM
STY $2116  ;[$7E2116] ;
LDA #$02              ;
STA $420B  ;[$7E420B] ;
SEP #$02              ;
BEQ $02               ;} Strange unconditional jump o_O
BRA $FE               ;  Crash
REP #$20              ;
TYA                   ;
ORA #$0020            ;
STA $2116  ;[$7E2116] ;
SEP #$20              ;
STX $4315  ;[$7E4315] ;} DMA [$0982] bytes from $7E[$098A] to [$0986]20h VRAM
LDX $098A  ;[$7E098A] ;
STX $4312  ;[$7E4312] ;
LDA #$02              ;
STA $420B  ;[$7E420B] ;
RTS                   ;


;-----------------------------------------------------------


; 7E90 $80A176 Display the viewable part of the room

org !7E90

PHP                    ;  Push Processor Status Register 08 Stack (Push)
SEP #$20               ;  Set Processor Status Bits [Flags affected: all except b per operand] E2 Immediate
LDA $58    ;[$7E:0058] ;\ LDA Load Accumulator from Memory [Flags affected: n,z] A5 Direct Page
SEC                    ;| Set Carry Flag [Flags affected: c] 38 Implied
SBC $59    ;[$7E:0059] ;| Subtract with Borrow from Accumulator [Flags affected: n,v,z,c] E5 Direct Page
XBA                    ;} BG2 size = BG1 address - BG2 address (XBA Exchange B and A)
REP #$20               ;| Reset Processor Status Bits [Flags affected: all except b per operand] C2 Immediate
AND #$F800             ;| AND Accumulator With Memory [Flags affected: n,z] 29 Immediate
STA $098E  ;[$7E:098E] ;/ Store Accumulator to Memory [Flags affected: none] 8D Absolute
JSR $A4BB  ;[$80:A4BB] ;  Calculate BG and layer position blocks
LDX #$0000             ;  Load Index Register X from Memory [Flags affected: n,z] A2 Immediate  Used to measure the number of times the
                       ;  LOOP is executed, before exiting it
LDA $08F7              ;  How many blocks X Layer 1 is scrolled
CLC                    ;
ADC #$FFFB             ;
STA $08F7              ;  Now the left limit for Layer 1 was moved to -5 
LDA $0907              ;  How many blocks X BG1 is scrolled
CLC                    ;
ADC #$FFFB             ;
STA $0907              ;  Now the left limit for BG1 was moved to -5
LDA $08FB              ;  How many blocks X Layer 2 is scrolled
CLC                    ;
ADC #$FFFB             ;
STA $08FB              ;  Now the left limit for Layer 2 was moved to -5
LDA $090B              ;  How many blocks X BG2 is scrolled
CLC                    ;
ADC #$FFFB             ;
STA $090B              ;  Now the left limit for BG2 was moved to -5
        ; LOOP
PHX                    ;  it starts at zero and 1 is added every time this LOOP is executed, its value is saved here
LDA $08F7  ;[$7E:08F7] ;\ How many blocks X Layer 1 is scrolled
STA $0990  ;[$7E:0990] ;} Blocks to update X block = [layer 1 block X]
LDA $08F9  ;[$7E:08F9] ;\
STA $0992  ;[$7E:0992] ;} Blocks to update Y block = [layer 1 block Y]
LDA $0907  ;[$7E:0907] ;\
STA $0994  ;[$7E:0994] ;} VRAM blocks to update X block = [BG1 block X]
LDA $0909  ;[$7E:0909] ;\
STA $0996  ;[$7E:0996] ;} VRAM blocks to update Y block = [BG1 block Y]
LDA $08F7  ;[$7E:08F7] ;\ $7E:08F7 Stores tile of the room scrolling, 0 all to the left   How many blocks X Layer 1 is scrolled
CMP #$0000             ;}
BMI $15                ;
LDA $07A8  ;[$7E:07A8] ;  $7E:07A8 Stores width of the room in pixels (100h,200h,300h,400h or more)
LSR A                  ;  Divide by 2
LSR A                  ;  Divide by 2
LSR A                  ;  Divide by 2
LSR A                  ;  Divide by 2
SEC                    ; 
SBC #$0001             ;  Substrat #$0001h (0001d)
CMP $08F7  ;[$7E:08F7] ;  Compare A with tile of the room,   Position of block X Layer 1
BMI $05                ;
JSR $A9DB  ;[$80:A9DB] ;  Update level data column Original
BRA $03                ;
JSR !7205  ;[$80:F205] ;  Update level data column Modified to load Black Blocks
LDA $091B  ;[$7E:091B] ;\
LSR A                  ;} If using custom layer 2:
BCS $38                ;/ BCS $38
LDA $08FB  ;[$7E:08FB] ;\
STA $0990  ;[$7E:0990] ;} Blocks to update X block = [layer 2 block X]
LDA $08FD  ;[$7E:08FD] ;\
STA $0992  ;[$7E:0992] ;} Blocks to update Y block = [layer 2 block Y]
LDA $090B  ;[$7E:090B] ;\
STA $0994  ;[$7E:0994] ;} VRAM blocks to update X block = [BG2 block X]
LDA $090D  ;[$7E:090D] ;\
STA $0996  ;[$7E:0996] ;} VRAM blocks to update Y block = [BG2 block Y]
LDA $08F7  ;[$7E:08F7] ;\ $7E:08F7 Stores tile of the room scrolling, 0 all to the left   How many blocks X Layer 1 is scrolled
CMP #$0000             ;}
BMI $15                ; 
LDA $07A8  ;[$7E:07A8] ;  $7E:07A8 Stores width of the room in pixels (100h,200h,300h,400h or more)
LSR A                  ;  Divide by 2
LSR A                  ;  Divide by 2
LSR A                  ;  Divide by 2
LSR A                  ;  Divide by 2
SEC                    ; 
SBC #$0001             ;  Substrat #$0001h (0001d)
CMP $08F7  ;[$7E:08F7] ;  Compare A with tile of the room,   Position of block X Layer 1
BMI $05                ; 
JSR $A9D6  ;[$80:A9D6] ;  Update background data column Original
BRA $03
JSR !7200  ;[$80:F200] ;  Update background data column Modified to load Black Blocks
JSL $808C83;[$80:8C83] ;  Handle VRAM write table and scrolling DMAs
PLX                    ;
INC $08F7  ;[$7E:08F7] ;  Increment layer 1 block X
INC $0907  ;[$7E:0907] ;  Increment BG1 block X
INC $08FB  ;[$7E:08FB] ;  Increment layer 2 block X
INC $090B  ;[$7E:090B] ;  Increment BG2 block X
INX                    ;\
CPX #$001B             ;} Repeat 26 more times (I changed from 16 to 27) (Is executed 27d times, which are the columns that are updated)
BPL $03                ;| 
BRL $FF6F              ;/ [BRL -145d] BNE $AC  ACh=172d  172-256= -84  -84=FFAC  FFFFFFFFFFFFFFAC-3D=FFFFFFFFFFFFFF6F
PLP                    ; 
RTL                    ;


;-----------------------------------------------------------


; 7F90 $80AF02 Door transition down and up, flag row 0

org !7F90

; Subroutine 80FF90 Save Flag on Doors Up and Down

LDA $0925  ;[$7E:0925] ; 
BNE $07                ;  If number of times screen has scrolled is not 0 (go to LDX $0925)
LDA #$0001             ;
STA $7EFFF8 ;[$7E:FFF8];  set the flag to 0001
LDX $0925  ;[$7E:0925] ;
RTS                    ;


;-----------------------------------------------------------








;-----------------------------------------------------------


; 74B0 $80AEC2 Door transition - left Modified - Fast Doors

;First change JMP to Free space

org $80AEC2
JMP !74B0


;-----------------------------------------------------------


; 7520 $80AE7E Door transition - right Modified - Fast Doors

;First change JMP to Free space

org $80AE7E
JMP !7520


;-----------------------------------------------------------


; 75A0 $80A3DF Update BG graphics when scrolling Modified Scrolling left & right, saves up & down

;First change JMP to Free space

org $80A3DF
JMP !75A0


;-----------------------------------------------------------


; 7730 First and second round of $80AB78 Update levelbackground data row

;First change JMP to Free space

org $80AB78
JMP !7730


;-----------------------------------------------------------


; 7C10 $808DAC Execute vertical scrolling DMAs 2 rounds

;First change JMP to Free space

org $808DAC
JMP !7C10


;-----------------------------------------------------------


; 7E90 $80A176 Display the viewable part of the room

;First change JMP to Free space

org $80A176
JMP !7E90


;-----------------------------------------------------------


; 7F90 $80AF02 Door transition down and up, flag row 0
; Subroutine 80FF90 Save Flag on Doors Up and Down

;$80AF02: Door transition - down

org $80AF02
JSR !7F90


;$80AF89: Door transition - up

org $80AF89
JSR !7F90


;-----------------------------------------------------------





;-----------------------------------------------------------


; Draw PLMs 20DAA ROM Address  Bank $848DAA


; First change to $848DAA: Draw PLM, Doors of the Right 16:9 area

org $849033  ; $21033
DB $1B


; Second change to $848DAA: Draw PLM, Doors of the Left 16:9 area, JMP to Free space

org $849027  ; $21027
JMP !27E00  ;[$84:FE00] ;4C 00 FE         ;  Jump to $84FE00 ; !27E00 = $FE00
NOP                     ;EA               ;  No Operation


; Third change to $848DAA: Draw PLM, Doors of the Left 16:9 area

org $840000+!27E00  ; !27E00 = $FE00
LSR A                   ;4A               ;  Divide by 2
LSR A                   ;4A               ;  Divide by 2
LSR A                   ;4A               ;  Divide by 2
LSR A                   ;4A               ;  Divide by 2
SEC                     ;38               ;  Set Carry for Substraction
SBC #$0005              ;E9 05 00         ;  Substrat #$0005h (0005d)
JMP $902B  ;[$84:902B]  ;4C 2B 90         ;  Jump to $84902B

;Animation of doors of the right and left are updated when shooting on the 16:9 area


; Fourth change to $848DAA: Draw PLM, Missiles, Power Bombs, E-tanks and other items and blocks, Left Part of the 16:9 area

org $848DF7  ; $20DF7
LSR A                   ;4A               ;  Divide by 2
LSR A                   ;4A               ;  Divide by 2
LSR A                   ;4A               ;  Divide by 2
LSR A                   ;4A               ;  Divide by 2
CLC                     ;18               ;  Clear Carry for Addition
ADC #$FFFA              ;69 FA FF         ;  Add #$FFFAh (-0006d) to Accumulator
NOP                     ;EA               ;  NOP


; Fifth change to $848DAA: Draw PLM, Missiles, Power Bombs, E-tanks and other items and blocks, Right Part of the 16:9 area

org $848E27  ; $20E27
DB $1B


;Process animation of items and blocks that respawn throughout the 16:9 area


;-----------------------------------------------------------





;-----------------------------------------------------------


; Kazuto's - Doors Transitions Speed

org $80AE9D
	dw !S
org $80AEA7
	dw !S
org $80AEB6
	dw !X
org $80AEE1
	dw !S
org $80AEEB
	dw !S
org $80AEFA
	dw !X
org $80AF45
	dw !Y
org $80AF64
	dw !S
org $80AF6E
	dw !S
org $80AF7D
	dw !Y
org $80AFE6
	dw !S
org $80AFF0
	dw !S
org $80AFF6
	dw !C
org $80B02A
	dw !Y
org $82D962
	dw !F

org $82DE50
	BPL $0F
	LDA $0791
	ROR A
	ROR A
	BCS $05
	LDA #$00C8
	BRA $03
	LDA #$0180

;Uncomment one of the three following lines
;	LSR A		;Uncomment only if S equals $0002
;	NOP		;Uncomment only if S equals $0004
	ASL A		;Uncomment only if S equals $0008


;-----------------------------------------------------------


; Kazuto's - Pause Screen Speed

org $828D30
	dw !P
org $8291CB
	dw !P
org $8291EE
	dw !P
org $82937D
	dw !P
org $82A5CA
	dw !P
org $90EA6F
	dw !P


;-----------------------------------------------------------


; Kazuto's - Door Centering Speed

org $82E325	;Horizontal doors
	NOP
	PHP
	LDX #$0004
	PLP
	JSL SlideCode
org $82E339	;Vertical doors
	NOP
	PHP
	LDX #$0000
	PLP
	JSL SlideCode

org !FreeSpace  ;!FreeSpace = $82FC60	;Safe to move anywhere in ROM ($017C60)
SlideCode:
	SEP #$20
	BMI $09
	LDA $0911,X
	CMP.b #$00+!Speed
	BPL $12
	BRA $0A
	LDA $0911,X
	CMP.b #$00-!Speed
	BMI $10
	INC $0912,X	;These two lines handle odd
	STZ $0911,X	;screen scrolling distances
	REP #$20
	RTL

	REP #$20
	LDA.w #$0000-!Speed	;Screen is scrolling up or left
	BRA $05
	REP #$20
	LDA.w #$0000+!Speed	;Screen is scrolling down or right
	CLC
	ADC $0911,X
	STA $0911,X
	RTL


;-----------------------------------------------------------


; Lioran's 2x Elevator Speed + Down fix

;Free space: !freebytes
;Bank $A3FBC0  ;$A3FBC0  Free space required: 2Bh Bytes, From 11FBC0 to 11FBEA in ROM Addresses
;ROM 11FBC0

org $A39581
JSL !freebytes ;hijack the elevator transition code for down elevator cause just setting the velocity of that one cause unwanted visual effect.
NOP #$15

org !freebytes ;write your code at the free space

STZ $0799
LDA #$0008 : CMP $0998 : BNE ++  ;check game state is 08, if it is go on
STZ !freeram ;set free ram to 0
JMP +++
++
LDA #!timer : CMP !freeram : BEQ + ;if timer is not equal to !timer then increase by 1.
LDA !freeram : INC A : STA !freeram ;increase timer by 1
+++
LDA $0F7E, x ;samus y position ram position
CLC : ADC #!velocity ;move samus !velocity amount vertically down
STA $0F7E, x
+
RTL

;set other elevator velocity to !velocity aswell those dont need to have a fix to it

org $A395B0
DB !velocity

org $A395D2
DB !velocity

org $A395EC
DB !velocity


;-----------------------------------------------------------





;-----------------------------------------------------------


;Enemies Part 1


;-----------------------------------------------------------


;Norfair lavaquake rock

;!85FF30 = $85FF00  ;ROM Address $2FF00


;First change to $86BD2A: Enemy projectile - Norfair lavaquake rocks, Right Part of the 16:9 area

org $86BD37  ; $33D37 ROM Address
DB $4A


;Second change to $86BD2A: Enemy projectile - Norfair lavaquake rocks, Left Part of the 16:9 area, JMP to Free space

org $86BD2A  ; $33D2A ROM Address
JMP !85FF30  ;[$85:FF30] ;5C30FF85         ;  JML to $85FF30 ; !85FF30 = $85FF30


;Third change to $86BD2A: Enemy projectile - Norfair lavaquake rocks, Left Part of the 16:9 area

org !85FF30  ; !85FF30 = $85FF30

LDA $1A4B,x ;[$7E:1A4B] ;BD 4B 1A         ;  LDA addr,X  BD  Absolute Indexed,X
CLC                     ;18               ;  CLC Clear Carry for Addition
ADC #$004A              ;69 4A 00         ;  ADC #const  69  Immediate
CMP $0911               ;CD 11 09         ;  CMP addr    CD  Absolute
JML $86BD30             ;5C 30 BD 86      ;  JML to $86BD30

;The lavaquake rocks in Norfair are seen in the entire 16:9 area


;-----------------------------------------------------------


; Spike shooting plant spikes

;!85FF50 = !85FF30+$20


;First change to $86DACE: Enemy projectile - Spike shooting plant spikes, Right Part of the 16:9 area

org $86DADB  ; $35ADB ROM Address
DB $4A
;DW $014A


;Second change to $86DACE: Enemy projectile - Spike shooting plant spikes, Left Part of the 16:9 area, JMP to Free space

org $86DACE  ; $35ACE ROM Address
JML !85FF50  ;[$85:FF50] ;5C50FF85         ;  JML to $85FF50 ; !85FF50 = !85FF30+$20


;Third change to $86DACE: Enemy projectile - Spike shooting plant spikes, Left Part of the 16:9 area

org !85FF50  ; !85FF50 = !85FF30+$20

LDA $1A4B,x ;[$7E:1A4B] ;BD 4B 1A         ;  LDA addr,X  BD  Absolute Indexed,X
CLC                     ;18               ;  CLC Clear Carry for Addition
ADC #$004A              ;69 4A 00         ;  ADC #const  69  Immediate
CMP $0911               ;CD 11 09         ;  CMP addr    CD  Absolute
JML $86DAD4             ;5C D4 DA 86      ;  JML to $86DAD4

;Spike shooting plant spikes are seen in the entire 16:9 area


;-----------------------------------------------------------


; Enemy projectiles - Fune & Namihe fireball

;!85FF70 = !85FF30+$40


;First change to $86:DBC2 Enemy projectiles - Fune & Namihe fireball, Right Part of the 16:9 area

org $86DBCF  ; $35BCF ROM Address
DB $58


;Second change to $86:DBC2 Enemy projectiles - Fune & Namihe fireball, Left Part of the 16:9 area, JMP to Free space

org $86DBC2  ; $35BC2 ROM Address
JML !85FF70  ;[$85:FF70] ;5C70FF85         ;  JML to $85FF70 ; !85FF70 = !85FF30+$40


;Third change to $86:DBC2 Enemy projectiles - Fune & Namihe fireball, Left Part of the 16:9 area

org !85FF70  ; !85FF70 = !85FF30+$40

LDA $1A4B,x ;[$7E:1A4B] ;BD 4B 1A         ;  LDA addr,X  BD  Absolute Indexed,X
CLC                     ;18               ;  CLC Clear Carry for Addition
ADC #$0058              ;69 58 00         ;  ADC #const  69  Immediate
CMP $0911               ;CD 11 09         ;  CMP addr    CD  Absolute
JML $86DBC8             ;5C C8 DB 86      ;  JML to $86DBC8

;Fune & Namihe fireball projectiles are seen in the entire 16:9 area


;-----------------------------------------------------------


; Enemy projectile - lava thrown by Magdollite

;!85FF90 = !85FF30+$60


;First change to $86E0B0: Enemy projectile - lava thrown by Magdollite, Right Part of the 16:9 area

org $86E0BD  ; $360BD ROM Address
DB $4A


;Second change to $86E0B0: Enemy projectile - lava thrown by Magdollite, Left Part of the 16:9 area, JMP to Free space

org $86E0B0  ; $360B0 ROM Address
JML !85FF90  ;[$85:FF90] ;5C90FF85         ;  JML to $85FF90 ; !85FF90 = !85FF30+$60


;Third change to $86E0B0: Enemy projectile - lava thrown by Magdollite, Left Part of the 16:9 area

org !85FF90  ; !85FF90 = !85FF30+$60

LDA $1A4B,x ;[$7E:1A4B] ;BD 4B 1A         ;  LDA addr,X  BD  Absolute Indexed,X
CLC                     ;18               ;  CLC Clear Carry for Addition
ADC #$004A              ;69 4A 00         ;  ADC #const  69  Immediate
CMP $0911               ;CD 11 09         ;  CMP addr    CD  Absolute
JML $86E0B6             ;5C B6 E0 86      ;  JML to $86E0B6

;Fire that launches Magdollite is seen in the entire 16:9 area


;-----------------------------------------------------------


; Check if enemy projectile is off screen

;!85FFB0 = !85FF30+$80


;First change to $86E6E0: Check if enemy projectile is off screen, Right Part of the 16:9 area

org $86E6EF  ; $366EF ROM Address
DB $4A


;Second change to $86E6E0: Check if enemy projectile is off screen, Left Part of the 16:9 area, JMP to Free space

org $86E6E0  ; $366E0 ROM Address
JML !85FFB0  ;[$85:FFB0] ;5CB0FF85         ;  JML to $85FFB0 ; !85FFB0 = !85FF30+$80


;Third change to $86E6E0: Check if enemy projectile is off screen, Left Part of the 16:9 area

org !85FFB0  ; !85FFB0 = !85FF30+$80

PHX                     ;DA               ;  PHX  DA  Stack (Push)
PHY                     ;5A               ;  PHY  5A  Stack (Push)
LDA $1A4B,x ;[$7E:1A4B] ;BD 4B 1A         ;  LDA addr,X  BD  Absolute Indexed,X
CLC                     ;18               ;  CLC Clear Carry for Addition
ADC #$004A              ;69 4A 00         ;  ADC #const  69  Immediate
JML $86E6E5             ;5C E5 E6 86      ;  JML to $86E6E5

;Skrees and Metarees particles, when you defeat them they explode in 4 little parts, these are seen in the entire 16:9 area
;Modified Subroutine so that Space Pirates rays can be seen throughout the 16:9 area


;-----------------------------------------------------------


; Enemy projectile - Botwoon's projectile

;!85FFD0 = !85FF30+$A0


;First change to $86EC18: Enemy projectile - Botwoon's projectile, Right Part of the 16:9 area

org $86EC25  ; $36C25 ROM Address
DB $4A


;Second change to $86EC18: Enemy projectile - Botwoon's projectile, Left Part of the 16:9 area, JMP to Free space

org $86EC18  ; $36C18
JML !85FFD0  ;[$85:FFD0] ;5CD0FF85         ;  JML to $85FFD0 ; !85FFD0 = !85FF30+$A0


;Third change to $86EC18: Enemy projectile - Botwoon's projectile, Left Part of the 16:9 area

org !85FFD0  ; !85FFD0 = !85FF30+$A0

LDA $1A4B,x ;[$7E:1A4B] ;BD 4B 1A         ;  LDA addr,X  BD  Absolute Indexed,X
CLC                     ;18               ;  CLC Clear Carry for Addition
ADC #$004A              ;69 4A 00         ;  ADC #const  69  Immediate
CMP $0911               ;CD 11 09         ;  CMP addr    CD  Absolute
JML $86EC1E             ;5C 1E EC 86      ;  JML to $86EC1E

;Projectiles launched by Botwoon are seen in the entire 16:9 area


;-----------------------------------------------------------





;-----------------------------------------------------------


;Enemies Part 2


;-----------------------------------------------------------


; Determine which enemies to process

;!9FFF60 = $9FFF00  ;ROM Address $FFFF0


;First change to $A08EB6: Determine which enemies to process, Left Part of the 16:9 area 

org $A08F11  ; $100F11 ROM Address
DB $4A


;Second change to $A08EB6: Determine which enemies to process, Right Part of the 16:9 area, JMP to Free space

org $A08F00  ; $100F00 ROM Address
JML !9FFF60  ;[$9F:FF60] ;5C60FF9F         ;  JML to $9FFF60 ; !9FFF60 = $9FFF60


;Third change to $A08EB6: Determine which enemies to process, Right Part of the 16:9 area

org !9FFF60  ; !9FFF60 = $9FFF60

LDA $0F7A,x ;[$7E:0F7A] ;BD 7A 0F         ;  LDA addr,X  BD  Absolute Indexed,X
CLC                     ;18               ;  CLC Clear Carry for Addition
ADC $0F82,x ;[$7E:0F82] ;7D 82 0F         ;  ADC addr,X  7D  Absolute Indexed,X
CLC                     ;18               ;  CLC Clear Carry for Addition
ADC #$004A              ;69 4A 00         ;  ADC #const  69  Immediate
CMP $0911               ;CD 11 09         ;  CMP addr    CD  Absolute
BMI $04                 ;30 04            ;  BMI nearlabel  30  Program Counter Relative
JML $A08F0C             ;5C 0C 8F A0      ;  JML to $A08F0C
JML $A08F54             ;5C 54 8F A0      ;  JML to $A08F54

;You can defeat enemies in the 16:9 area, even if they are not seen, the game processes all the enemies that are in the 16:9 area


;-----------------------------------------------------------


; Gamet, the red enemies that divided into many

;!9FFF90 = !9FFF60+$30


;First change to $A0AD70 Determines if enemy is on screen or not, Right Part of the 16:9 area

org $A0AD80  ; $102D80 ROM Address
DB $4A


;Second change to $A0AD70 Determines if enemy is on screen or not, Left Part of the 16:9 area, JMP to Free space

org $A0AD70  ; $102D70 ROM Address
JML !9FFF90  ;[$9F:FF90] ;5C90FF9F         ;  JML to $9FFF90 ; !9FFF90 = !9FFF60+$30


;Third change to $A0AD70 Determines if enemy is on screen or not, Left Part of the 16:9 area

org !9FFF90  ; !9FFF90 = !9FFF60+$30

LDX $0E54   ;[$7E:0E54] ;AE 54 0E         ;  LDX addr  AE  Absolute
LDA $0F7A,x ;[$7E:1A4B] ;BD 7A 0F         ;  LDA addr,X  BD  Absolute Indexed,X
CLC                     ;18               ;  CLC Clear Carry for Addition
ADC #$004A              ;69 4A 00         ;  ADC #const  69  Immediate
JML $A0AD76             ;5C 76 AD A0      ;  JML to $A0AD76

;Gamet, the red enemies that divided into many, are seen in the entire 16:9 area


;-----------------------------------------------------------


; Determines if enemies with normal sprites are off screen, Draw enemies perfectly throughout the 16:9 area

;!9FFFB0 = !9FFF60+$50


;First change to $A0ADE7 Determines if enemies with normal sprites are off screen, Right Part of the 16:9 area

org $A0ADFC  ; $102DFC ROM Address
DB $4A


;Second change to $A0ADE7 Determines if enemies with normal sprites are off screen, Left Part of the 16:9 area, JMP to Free space

org $A0ADE7  ; $102DE7 ROM Address
JML !9FFFB0  ;[$9F:FFB0] ;5CB0FF9F         ;  JML to $9FFFB0 ; !9FFFB0 = !9FFF60+$50


;Third change to $A0ADE7 Determines if enemies with normal sprites are off screen, Left Part of the 16:9 area

org !9FFFB0  ; !9FFFB0 = !9FFF60+$50

PHX                     ;DA               ;  PHX  DA  Stack (Push)
LDX $0E54   ;[$7E:0E54] ;AE 54 0E         ;  LDX addr  AE  Absolute
LDA $0F7A,x ;[$7E:0F7A] ;BD 7A 0F         ;  LDA addr,X  BD  Absolute Indexed,X
CLC                     ;18               ;  CLC Clear Carry for Addition
ADC $0F82,x ;[$7E:0F82] ;7D 82 0F         ;  ADC addr,X  7D  Absolute Indexed,X
CLC                     ;18               ;  CLC Clear Carry for Addition
ADC #$004A              ;69 4A 00         ;  ADC #const  69  Immediate
JML $A0ADF2             ;5C F2 AD A0      ;  JML to $A0ADF2

;Draw enemies perfectly throughout the 16:9 area


;-----------------------------------------------------------


; Deternine if Zeb and Zebbo are on screen

;!9FFFE0 = !9FFF60+$80


;$A0C18E Deternine if Zeb and Zebbo are on screen  

;First change to $A0C18E: Deternine if Zeb and Zebbo are on screen, Right Part of the 16:9 area

org $A0C19F  ; $10419F ROM Address
DB $94


;Second change to $A0C18E Deternine if Zeb and Zebbo are on screen, Left Part of the 16:9 area, JMP to Free space

org $A0C198  ; $104198 ROM Address
JML !9FFFE0  ;[$9F:FFE0] ;5CE0FF9F         ;  JML to $9FFFE0 ; !9FFFE0 = !9FFF60+$80


;Third change to $A0C18E Deternine if Zeb and Zebbo are on screen, Left Part of the 16:9 area

org !9FFFE0  ; !9FFFE0 = !9FFF60+$80

SBC $0911   ;[$7E:0911] ;ED 11 09         ;  SBC addr  ED  Absolute
CLC                     ;18               ;  CLC Clear Carry for Addition
ADC #$004A              ;69 4A 00         ;  ADC #const  69  Immediate
BMI $04                 ;30 04            ;  BMI nearlabel  30  Program Counter Relative
JML $A0C19D             ;5C 9D C1 A0      ;  JML to $A0C19D
JML $A0C1AC             ;5C AC C1 A0      ;  JML to $A0C1AC

;Zeb and Zebbo are seen in the entire 16:9 area


;-----------------------------------------------------------


;$A2B8D3 Determine if Rinka is on screen

org $A2B8EE  ; $1138EE ROM Address
DB $5A

org $A2B8F7  ; $1138F7 ROM Address
DB $B4

;Rinka is seen in the entire 16:9 area


;-----------------------------------------------------------


;$B4BD32: Draw sprite objects, Explosions of Space Pirates when defeating them on the Right and Left Side of 16:9 area

org $B4BD52  ; $1A3D52 ROM Address
DB $5A

org $B4BD57  ; $1A3D57 ROM Address
DB $B4

;Draw enemies perfectly throughout the 16:9 area


;-----------------------------------------------------------





;-----------------------------------------------------------


;Banks and ROM Addresses

;808000 000000  908000 080000  A08000 100000  B08000 180000
;818000 008000  918000 088000  A18000 108000  B18000 188000
;828000 010000  928000 090000  A28000 110000  B28000 190000
;838000 018000  938000 098000  A38000 118000  B38000 198000
;848000 020000  948000 0A0000  A48000 120000  B48000 1A0000
;858000 028000  958000 0A8000  A58000 128000  B58000 1A8000
;868000 030000  968000 0B0000  A68000 130000  B68000 1B0000
;878000 038000  978000 0B8000  A78000 138000  B78000 1B8000
;888000 040000  988000 0C0000  A88000 140000  B88000 1C0000
;898000 048000  998000 0C8000  A98000 148000  B98000 1C8000
;8A8000 050000  9A8000 0D0000  AA8000 150000  BA8000 1D0000
;8B8000 058000  9B8000 0D8000  AB8000 158000  BB8000 1D8000
;8C8000 060000  9C8000 0E0000  AC8000 160000  BC8000 1E0000
;8D8000 068000  9D8000 0E8000  AD8000 168000  BD8000 1E8000
;8E8000 070000  9E8000 0F0000  AE8000 170000  BE8000 1F0000
;8F8000 078000  9F8000 0F8000  AF8000 178000  BF8000 1F8000

;C08000 200000  D08000 280000  When the Bank is even, these
;C18000 208000  D18000 288000  are the values for the last
;C28000 210000  D28000 290000     part of the addresses
;C38000 218000  D38000 298000
;C48000 220000  D48000 2A0000          8000 0000
;C58000 228000  D58000 2A8000          9000 1000
;C68000 230000  D68000 2B0000          A000 2000
;C78000 238000  D78000 2B8000          B000 3000
;C88000 240000  D88000 2C0000          C000 4000
;C98000 248000  D98000 2C8000          D000 5000
;CA8000 250000  DA8000 2D0000          E000 6000
;CB8000 258000  DB8000 2D8000          F000 7000
;CC8000 260000  DC8000 2E0000
;CD8000 268000  DD8000 2E8000     Example 848000 020000 
;CE8000 270000  DE8000 2F0000     Example 84DDAA 025DAA
;CF8000 278000  DF8000 2F8000     Example 87F3A0 03F3A0


;-----------------------------------------------------------





;-----------------------------------------------------------





;-----------------------------------------------------------




